"""     Template wrapper to display basic unit:     Box.py

The program SALOME is required to run this script. SALOME is available from https://www.salome-platform.org
Tested with SALOME 9.3.0 and 9.5.0.

 - before you start: adjust the PATH in 'mypath' to point to the localation of the folder 'shapes' on your local drive

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

mypath = "C:/Users/supplement/basic-blocks/shapes/"

import sys, math, time
sys.path.insert( 0, mypath)
start_time = time.time()

# GEOMETRY
inputgeo = []	# container for input variables geometry
## offset in x, y, z
inputgeo.append(0.0) # origin coordinate x	/ 0
inputgeo.append(0.0) # origin coordinate y	/ 1
inputgeo.append(0.0) # origin coordinate z	/ 2

#            [  0        1       2       3         4        5 ]
myfacenames= [ "bottom", "top", "front", "back", "left", "right"]
myfacegroups = [ [],[],[],[],[],[] ] # this must be the same number as myfacenames (it is used to match the facegroup with the correct name from myfacenames)
myvolname = "Box 1"    # to assign a name to the volume group (if needed)

# intitialise Salome
import salome
salome.salome_init()
import GEOM
from salome.geom import geomBuilder
geompy = geomBuilder.New()
import SMESH
from salome.smesh import smeshBuilder
smesh = smeshBuilder.New()

# load basic unit
import Box

print( "   Starting script ..." )

# create object (geometry)
mybox = []   # empty array to collect points in space
rot = 0 # rotation in space
mx = 10.0e-3
my = 10.0e-3
mz = 10.0e-3
Box.DefPts(geompy, math, mx, my, mz, inputgeo[0], inputgeo[1], inputgeo[2], rot, mybox)
myobject = Box.Make(geompy, mybox)
## define faces
faces = [0, 1, 2, 3, 4, 5]
grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
## create mesh
hexa = smesh.Mesh(myobject) 
segx = 6
segy = 6
segz = 6
Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, segy)	# compute mesh
# copy groups from geometry into mesh (This seems to be the easiest way to define face and volume groups, which are needed later for boundary definition or porous media definition in the CFD simulation.)
i = 0
for a in grouplist:	
	if i == 0:
		hexa.GroupOnGeom(a, myvolname)
	else:
		hexa.GroupOnGeom(a, myfacenames[i-1])
	i = i + 1

salome.sg.updateObjBrowser()
print( "   ... done in %.1f seconds." % (time.time() - start_time) )