"""     Bend.py

The program SALOME is required to run this script. SALOME is available from https://www.salome-platform.org
Tested with SALOME 9.3.0 and 9.5.0.

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

#############################################################################################
### FUNCTION to define all points for one bend
#############################################################################################
def DefPts(geompy, math, x, z, y, x0, y0, z0, myangle, rot, mybox):
	# compute all 20 points for a 90 degree bend ( and add them to a list -> mybox (mybox must be an empty list !!) 
	# x = outer radius
	# y = inner radius
	rot = math.radians(rot) # roation of the whole shape
	q = math.radians(myangle) # full range
	h = math.radians( int(myangle / 2) ) # half range 
	# circular coordinates, rot should be either: 0, 90, 180, 270
	# x = x * math.cos(rot)
	# y = x * math.sin(rot)
	
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True
	mydisplayc = 0
	
	## arc0 / outer arc
	a = x0 + x * math.cos(rot)
	b = y0 + x * math.sin(rot)
	c = z0 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + x * math.cos(rot + h)
	b = y0 + x * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + x * math.cos(rot + q)
	b = y0 + x * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	## arc1 / copy in z direction
	a = x0 + x * math.cos(rot)
	b = y0 + x * math.sin(rot)
	c = z0 + z
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + x * math.cos(rot + h)
	b = y0 + x * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + x * math.cos(rot + q)
	b = y0 + x * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	## arc2 / inner arc
	a = x0 + y * math.cos(rot)
	b = y0 + y * math.sin(rot)
	c = z0 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + y * math.cos(rot + h)
	b = y0 + y * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + y * math.cos(rot + q)
	b = y0 + y * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	## arc3 / copy in z direction
	a = x0 + y * math.cos(rot)
	b = y0 + y * math.sin(rot)
	c = z0 + z
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + y * math.cos(rot + h)
	b = y0 + y * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + y * math.cos(rot + q)
	b = y0 + y * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	# extra points for face detection (later)
	mydist = ( x - y ) * 0.5
	a = x0 + y * math.cos(rot + h) * 1.5
	b = y0 + ( y + mydist ) * math.sin(rot + h) 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	c = z0 
	a = x0 + y * math.cos(rot + h) * 1.5
	b = y0 + ( y + mydist ) * math.sin(rot + h) 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	c = z0 + 0.5 * z 
	a = x0 + x * math.cos(rot + h)
	b = y0 + x * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + y * math.cos(rot + h)
	b = y0 + y * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
############################################################################################
### FUNCTION to create the bend from a list of points
#############################################################################################
def Make(geompy, mybox):
	### make a 90 degree bend
	# create 4 arcs
	# create 8 edges
	# create all faces
	# make a SOLID from all faces	
	myarc = []
	myedge = []
	myface = []
	
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True
	mydisplayc = 0
	
	#######################################################
	### create arcs from three points
	#######################################################
	
	arc = geompy.MakeArc(mybox[0], mybox[1], mybox[2])
	myarc.append(arc)
	if mydisplay:
		ptname = "myArc0"
		geompy.addToStudy(arc, ptname)			# for visualization: add box to the geometry
	
	arc = geompy.MakeArc(mybox[3], mybox[4], mybox[5])
	myarc.append(arc)
	if mydisplay:
		ptname = "myArc1"
		geompy.addToStudy(arc, ptname)	
	
	arc = geompy.MakeArc(mybox[6], mybox[7], mybox[8])
	myarc.append(arc)
	if mydisplay:
		ptname = "myArc2"
		geompy.addToStudy(arc, ptname)	
		
	arc = geompy.MakeArc(mybox[9], mybox[10], mybox[11])
	myarc.append(arc)
	if mydisplay:
		ptname = "myArc3"
		geompy.addToStudy(arc, ptname)	
	
	#######################################################
	### create edges
	#######################################################
	
	edge = geompy.MakeEdge(mybox[0], mybox[6])
	myedge.append(edge)		# 0
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[6], mybox[9])
	myedge.append(edge)		# 1
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[3], mybox[9])
	myedge.append(edge)		# 2
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[0], mybox[3])
	myedge.append(edge)		# 3
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[2], mybox[8])
	myedge.append(edge)		# 4
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[2], mybox[5])
	myedge.append(edge)		# 5
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[5], mybox[11])
	myedge.append(edge)		# 6
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[8], mybox[11])
	myedge.append(edge)		# 7
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	#######################################################
	### create faces
	#######################################################
	
	wire = geompy.MakeWire( [myedge[0], myedge[1], myedge[2], myedge[3]] )	# first we make a WIRE 
	face = geompy.MakeFace(wire, True) # isPlanarWanted = True
	myface.append(face)		#  face 0
	
	wire = geompy.MakeWire( [myedge[4], myedge[5], myedge[6], myedge[7]] )	
	face = geompy.MakeFace(wire, True) # isPlanarWanted = True
	myface.append(face)		#  face 1
	
	wire = geompy.MakeWire( [myarc[0], myarc[2], myedge[0], myedge[4]] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 2
	
	wire = geompy.MakeWire( [myarc[1], myarc[3], myedge[2], myedge[6]] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 3
		
	face = geompy.MakePipe( myarc[0], myedge[3], 0 )	# we need to use MakePipe ( = extrusion of line to face) 
	myface.append(face)		#  face 4
	
	face = geompy.MakePipe( myarc[2], myedge[1], 0 )
	myface.append(face)		#  face 5
	
	#######################################################
	### create SOLID
	#######################################################
	
	# create a shell from a set of faces
	shell = geompy.MakeShell( myface )
	
	# create a solid, bounded by the given shell
	solid = geompy.MakeSolid( [shell] )
	
	return solid

#############################################################################################
### FUNCTION to make MESH for Bend without gradient
#############################################################################################
def Mesh(geompy, mybox, mygeo, hexa, segwidth, segheight, segarc):
	hexa_1d = hexa.Segment()
	
	# x-direction
	myseg = segwidth
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[6])	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
		
	# z-direction 
	myseg = segheight
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[3])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()

	# arcs
	myseg = segarc
	edge = geompy.GetEdgeNearPoint(mygeo, mybox[1])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	hexa.Quadrangle()
	hexa.Hexahedron()
	hexa.Compute()		# compute mesh
	
#############################################################################################
def MeshTri(geompy, mybox, mygeo, hexa, segwidth, segheight, segarc):
	hexa_1d = hexa.Segment()
	
	
	# x-direction
	myseg = segwidth
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[6])	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
		
	# z-direction 
	myseg = segheight
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[3])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()

	# arcs
	myseg = segarc
	edge = geompy.GetEdgeNearPoint(mygeo, mybox[1])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	#algo_local.Propagation()
	edge = geompy.GetEdgeNearPoint(mygeo, mybox[4])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	
	myseg = int(segarc / 3)
	edge = geompy.GetEdgeNearPoint(mygeo, mybox[7])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	#algo_local.Propagation()
	edge = geompy.GetEdgeNearPoint(mygeo, mybox[10])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	
	algo2D = hexa.Triangle()
	#algo2D.MaxElementArea(800.0)
	algo3D = hexa.Tetrahedron()
	#algo3D.MaxElementVolume(900.0)
	hexa.Compute()
	
	
#############################################################################################
### FUNCTION to make groups for bend faces 
#############################################################################################
def Groups(geompy, mybox, mygeo, mygroups, myfacelist, myfacenames, myvolname):
	#: Create GROUPS to be addressed later as boundary for OpenFOAM
	# VOLUME GROUP is assigned if myvolname is not empty
	# FACE GROUPS are assigend if myfacelist is not empty
	# myfacelist = [ 1, 2, 3, 4, 0, 0 ] # this assigns the different groups to the 6 sides of the box
	# a value of -1 means this face should NOT be assigned (e. g. inflow direction)
	# faces are only available if no other object is joined at the respective face
	# this is the order:  BendFaces = [ bottom, top, outer, inner, front, back]
	mygroups2 = []
	
	if len(myvolname) > 0:
		group = geompy.CreateGroup(mygeo, geompy.ShapeType["SOLID"])
		id = geompy.GetSubShapeID(mygeo, mygeo)
		geompy.AddObject(group, id)
		geompy.addToStudyInFather(mygeo, group, myvolname)
		mygroups2.append(group)
	
	### FACE GROUPS
	if len(myfacelist) > 0:
		
		i = 0
		a = myfacelist[i]
		if a > (-1):			# if face is boundary (not linked to other face)
			# bottom
			face = geompy.GetFaceNearPoint(mygeo, mybox[13])
			mygroups[a].append( face )  # collect face into the list for: wall, inlet, outlet, ...
		
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# top
			face = geompy.GetFaceNearPoint(mygeo, mybox[12])
			mygroups[a].append( face ) 
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# outer
			face = geompy.GetFaceNearPoint(mygeo, mybox[14])
			mygroups[a].append( face ) 
		
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# outer
			face = geompy.GetFaceNearPoint(mygeo, mybox[15])
			mygroups[a].append( face ) 
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# front
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[3], mybox[6], mybox[9] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# front
			face = geompy.GetFaceByPoints(mygeo, mybox[2], mybox[5], mybox[8], mybox[11] )
			mygroups[a].append( face )
			
		
		i = 0
		for b in mygroups:
			group = geompy.CreateGroup(mygeo, geompy.ShapeType["FACE"])
			m = False
			for a in b:
				id = geompy.GetSubShapeID(mygeo, a)
				geompy.AddObject(group, id)
				m = True
			if m:
				geompy.addToStudyInFather(mygeo, group, myfacenames[i])
				mygroups2.append(group)
			i = i + 1
	
	return mygroups2