"""     BendRecGrad.py

The program SALOME is required to run this script. SALOME is available from https://www.salome-platform.org
Tested with SALOME 9.3.0 and 9.5.0.

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

#############################################################################################
### FUNCTION to define all points for one bend
#############################################################################################
def DefPts(geompy, math, x, z, y, x0, y0, z0, rot, mybox):
	# compute all 20 points for a 90 degree bend ( and add them to a list -> mybox (mybox must be an empty list !!) 
	# x = total length
	# y = inner radius
	# z = height
	myangle = 90 
	rot = math.radians(rot) # roation of the whole shape
	q = math.radians(myangle) # full range
	h = math.radians( int(myangle / 2) ) # half range 
	mydia = math.sqrt( 2.0 * x * x )	 # diagonal
	# circular coordinates, rot should be either: 0, 90, 180, 270
	# x = x * math.cos(rot)
	# y = x * math.sin(rot)
	
	mydisplay = False # for visualization: add box to the geometry
	mydisplayc = 0
	
	## arc0 / outer arc
	a = x0 + x * math.cos(rot)
	b = y0 + x * math.sin(rot)
	c = z0 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + mydia * math.cos(rot + h)
	b = y0 + mydia * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + x * math.cos(rot + q)
	b = y0 + x * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	## arc1 / copy in z direction
	a = x0 + x * math.cos(rot)
	b = y0 + x * math.sin(rot)
	c = z0 + z
	pt = geompy.MakeVertex ( a, b, c )	# 3
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + mydia * math.cos(rot + h)
	b = y0 + mydia * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + x * math.cos(rot + q)
	b = y0 + x * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	## arc2 / inner arc
	a = x0 + y * math.cos(rot)
	b = y0 + y * math.sin(rot)
	c = z0 
	pt = geompy.MakeVertex ( a, b, c )	# 6
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + y * math.cos(rot + h)
	b = y0 + y * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + y * math.cos(rot + q)
	b = y0 + y * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	## arc3 / copy in z direction
	a = x0 + y * math.cos(rot)
	b = y0 + y * math.sin(rot)
	c = z0 + z
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + y * math.cos(rot + h)
	b = y0 + y * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + y * math.cos(rot + q)
	b = y0 + y * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	# extra points for face detection 
	mydist = ( x - y ) * 0.5
	a = x0 + y * math.cos(rot + h) * 1.5
	b = y0 + ( y + mydist ) * math.sin(rot + h) 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	c = z0 
	a = x0 + y * math.cos(rot + h) * 1.5
	b = y0 + ( y + mydist ) * math.sin(rot + h) 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + y * math.cos(rot + h)
	b = y0 + y * math.sin(rot + h)
	c = z0 + 0.5 * z 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	### points at half distance
	hx = 0.5 * (x -y)
	
	a = x0 + (y + hx) * math.cos(rot)
	b = y0 + (y + hx) * math.sin(rot)
	c = z0
	pt = geompy.MakeVertex ( a, b, c )	# 15
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	c = z0 + z * 0.5
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	c = z0 + z 
	pt = geompy.MakeVertex ( a, b, c )	# 17
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + (y + hx) * math.cos(rot + q)
	b = y0 + (y + hx) * math.sin(rot + q)
	c = z0
	pt = geompy.MakeVertex ( a, b, c )	# 18
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	c = z0 + z * 0.5
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	c = z0 + z 
	pt = geompy.MakeVertex ( a, b, c )	# 20
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + y * math.cos(rot)
	b = y0 + y * math.sin(rot)
	c = z0 + 0.5 * z 
	pt = geompy.MakeVertex ( a, b, c )	# 21
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + x * math.cos(rot)
	b = y0 + x * math.sin(rot)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + y * math.cos(rot + q)
	b = y0 + y * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )	 # 23
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + x * math.cos(rot + q)
	b = y0 + x * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )	 
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + mydia * math.cos(rot + h)
	b = y0 + mydia * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )	# 25
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
############################################################################################
### FUNCTION to create the bend from a list of points
#############################################################################################
def Make(geompy, mybox):
	### make a 90 degree bend
	# create 4 arcs
	# create 8 edges
	# create all faces
	# make a SOLID from all faces	
	myarc = []
	myedge = []
	myface = []
	
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True
	mydisplayc = 0
	
	#######################################################
	### create arcs from three points
	#######################################################
	arc = geompy.MakeArc(mybox[6], mybox[7], mybox[8])  # bottom arc
	myarc.append(arc)
	if mydisplay:
		test = 'arc' + str(mydisplayc)
		geompy.addToStudy(arc, test)
		mydisplayc = mydisplayc + 1
	
	arc = geompy.MakeArc(mybox[21], mybox[14], mybox[23]) # middle arc
	myarc.append(arc)
	if mydisplay:
		test = 'arc' + str(mydisplayc)
		geompy.addToStudy(arc, test)
		mydisplayc = mydisplayc + 1
		
	arc = geompy.MakeArc(mybox[9], mybox[10], mybox[11]) # top arc
	myarc.append(arc)
	if mydisplay:
		test = 'arc' + str(mydisplayc)
		geompy.addToStudy(arc, test)
		mydisplayc = mydisplayc + 1
	
	#######################################################
	### create edges
	#######################################################
	mydisplayc = 0
	edge = geompy.MakeEdge(mybox[6], mybox[15])
	myedge.append(edge)		# 0
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[15], mybox[0])
	myedge.append(edge)		# 1
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[0], mybox[22])
	myedge.append(edge)		# 2
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[22], mybox[3])
	myedge.append(edge)		# 3
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[3], mybox[17])
	myedge.append(edge)		# 4
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[17], mybox[9])
	myedge.append(edge)		# 5
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[9], mybox[21])
	myedge.append(edge)		# 6
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[21], mybox[6])
	myedge.append(edge)		# 7
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[21], mybox[16])
	myedge.append(edge)		# 8
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[16], mybox[22])
	myedge.append(edge)		# 9
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[0], mybox[1])
	myedge.append(edge)		# 10
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[1], mybox[25])
	myedge.append(edge)		# 11
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[25], mybox[4])
	myedge.append(edge)		# 12
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[4], mybox[3])
	myedge.append(edge)		# 13
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[22], mybox[25])
	myedge.append(edge)		# 14
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	###
	
	edge = geompy.MakeEdge(mybox[1], mybox[2])
	myedge.append(edge)		# 15
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[2], mybox[24])
	myedge.append(edge)		# 16
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[24], mybox[5])
	myedge.append(edge)		# 17
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[5], mybox[4])
	myedge.append(edge)		# 18
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[25], mybox[24])
	myedge.append(edge)		# 19
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	###
	
	edge = geompy.MakeEdge(mybox[2], mybox[18])
	myedge.append(edge)		# 20
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[18], mybox[8])
	myedge.append(edge)		# 21
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[8], mybox[23])
	myedge.append(edge)		# 22
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[23], mybox[11])
	myedge.append(edge)		# 23
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[11], mybox[20])
	myedge.append(edge)		# 24
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[20], mybox[5])
	myedge.append(edge)		# 25
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[24], mybox[19])
	myedge.append(edge)		# 26
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[19], mybox[23])
	myedge.append(edge)		# 27
	if mydisplay:
		test = 'edge' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	#######################################################
	### create faces
	#######################################################
	wire = geompy.MakeWire( [ myedge[0], myedge[1], myedge[2], myedge[7], myedge[8], myedge[9] ] )
	face = geompy.MakeFace(wire, True) # isPlanarWanted = True
	myface.append(face)		#  face 0 / front small bottom
	
	wire = geompy.MakeWire( [ myedge[8], myedge[9], myedge[3], myedge[4], myedge[5], myedge[6] ] )
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 1 / front small top
	
	wire = geompy.MakeWire( [ myedge[20], myedge[21], myedge[26], myedge[27], myedge[16], myedge[22] ] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 2 / left small bottom
	
	wire = geompy.MakeWire( [ myedge[26], myedge[27], myedge[24], myedge[25], myedge[17], myedge[23] ] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 3 / left small top
	
	wire = geompy.MakeWire( [ myedge[10], myedge[11], myedge[14], myedge[2] ] )
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 4 / right big bottom
	
	wire = geompy.MakeWire( [ myedge[14], myedge[12], myedge[13], myedge[3] ] )
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 5 / right big top
	
	wire = geompy.MakeWire( [ myedge[15], myedge[16], myedge[19], myedge[11], ] )
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 6 / back big bottom
	
	wire = geompy.MakeWire( [ myedge[19], myedge[17], myedge[18], myedge[12], ] )
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 7 / back big top
		
	wire = geompy.MakeWire( [ myarc[0], myedge[0], myedge[1], myedge[10], myedge[15], myedge[20], myedge[21] ] )
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 8 / bottom
	
	wire = geompy.MakeWire( [ myarc[2], myedge[4], myedge[5], myedge[13], myedge[18], myedge[24], myedge[25] ] )
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 9 / top
		
	face = geompy.MakePipe( myarc[0], myedge[7], 0 )	# we need to use MakePipe ( = extrusion of line to face) 
	myface.append(face)		#  face 10 / bend bottom
	
	face = geompy.MakePipe( myarc[2], myedge[6], 0 )
	myface.append(face)		#  face 11 / bend top
	
	#######################################################
	### create SOLID
	#######################################################
	
	# create a shell from a set of faces
	shell = geompy.MakeShell( myface )
	
	# create a solid, bounded by the given shell
	solid = geompy.MakeSolid( [shell] )
	
	return solid
#############################################################################################
### FUNCTION to make MESH without gradient / refinement
#############################################################################################
def Mesh(geompy, mybox, mygeo, hexa, segwidth, seggrad, segarc, segback):
	# segwidth = number of segments for front ( half rib )
	# segarc = number of segments for arc
	# segback = number of segments for large sides
	# seggrad = reference size for large mesh cells
	segmin = seggrad * 0.2
	segmax = seggrad * 0.8
		
	# front and left (small faces) 
	myseg = segwidth
	
	edge = geompy.GetEdge( mygeo, mybox[6], mybox[15] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[15], mybox[0] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[21], mybox[16] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[16], mybox[22] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[9], mybox[17] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[17], mybox[3] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	
	edge = geompy.GetEdge( mygeo, mybox[8], mybox[18] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[18], mybox[2] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[23], mybox[19] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[19], mybox[24] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[11], mybox[20] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[20], mybox[5] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	
	
	# right and back (big faces)
	myseg = segback
	
	edge = geompy.GetEdge( mygeo, mybox[0], mybox[1] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[22], mybox[25] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[3], mybox[4] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	
	edge = geompy.GetEdge( mygeo, mybox[1], mybox[2] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[24], mybox[25] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge( mygeo, mybox[4], mybox[5] )	
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	
		
	# z-direction 
	
	edge = geompy.GetEdge(mygeo, mybox[6], mybox[21])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmax, segmin )
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[22])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmin, segmax )
	edge = geompy.GetEdge(mygeo, mybox[21], mybox[9])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmin, segmax )
	edge = geompy.GetEdge(mygeo, mybox[22], mybox[3])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmax, segmin )
	
	edge = geompy.GetEdge(mygeo, mybox[2], mybox[24])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmin, segmax )
	edge = geompy.GetEdge(mygeo, mybox[8], mybox[23])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmin, segmax )
	edge = geompy.GetEdge(mygeo, mybox[24], mybox[5])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmax, segmin )
	edge = geompy.GetEdge(mygeo, mybox[23], mybox[11])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmax, segmin )
	
	edge = geompy.GetEdge(mygeo, mybox[1], mybox[25])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmin, segmax )
	edge = geompy.GetEdge(mygeo, mybox[25], mybox[4])	
	algo_local = hexa.Segment(edge)
	algo_local.Arithmetic1D( segmax, segmin )

	# arcs
	myseg = segarc
	edge = geompy.GetEdgeNearPoint(mygeo, mybox[7])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdgeNearPoint(mygeo, mybox[14])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdgeNearPoint(mygeo, mybox[10])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
		
		
	# Bottom face
	face = geompy.GetFaceNearPoint(mygeo, mybox[13])  # bottom
	hexa.Quadrangle(face)
	algo2D = hexa.Quadrangle()
	
	# Top face
	face = geompy.GetFaceNearPoint(mygeo, mybox[12])  # bottom
	hexa.Quadrangle(face)
	algo2D = hexa.Quadrangle()

	hexa.Quadrangle()
	hexa.Hexahedron()
	hexa.Compute()
	
#############################################################################################
### FUNCTION to make groups for bend faces 
#############################################################################################
def Groups(geompy, mybox, mygeo, mygroups, myfacelist, myfacenames, myvolname):
	#: Create GROUPS to be addressed later as boundary for OpenFOAM
	# VOLUME GROUP is assigned if myvolname is not empty
	# FACE GROUPS are assigend if myfacelist is not empty
	# myfacelist = [ 1, 2, 3, 4, 0, 0 ] # this assigns the different groups 
	# a value of -1 means this face should NOT be assigned (e. g. in flow direction)
	# this is the order:  BendRecFaces = [ bottom, top, left, right, inner, back1, back2 ]
	
	mygroups2 = []
	
	if len(myvolname) > 0:
		group = geompy.CreateGroup(mygeo, geompy.ShapeType["SOLID"])
		id = geompy.GetSubShapeID(mygeo, mygeo)
		geompy.AddObject(group, id)
		geompy.addToStudyInFather(mygeo, group, myvolname)
		mygroups2.append(group)
	
	### FACE GROUPS
	if len(myfacelist) > 0:
	
		i = 0
		a = myfacelist[i]
		if a > (-1):			# if face is boundary (not linked to other face)
			# bottom
			face = geompy.GetFaceNearPoint(mygeo, mybox[13])
			mygroups[a].append( face )  # collect face into the list for: wall, inlet, outlet, ...
		
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# top
			face = geompy.GetFaceNearPoint(mygeo, mybox[12])
			mygroups[a].append( face ) 
		
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# left
			face = geompy.GetFaceByPoints(mygeo, mybox[2], mybox[8], mybox[23], mybox[24] )
			mygroups[a].append( face )
			face = geompy.GetFaceByPoints(mygeo, mybox[23], mybox[24], mybox[5], mybox[11] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# right
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[6], mybox[21], mybox[22] )
			mygroups[a].append( face )
			face = geompy.GetFaceByPoints(mygeo, mybox[21], mybox[22], mybox[3], mybox[9] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# inner
			#face = geompy.GetFaceNearPoint(mygeo, mybox[14])
			face = geompy.GetFaceByPoints(mygeo, mybox[6], mybox[8], mybox[21], mybox[23] )
			mygroups[a].append( face )
			face = geompy.GetFaceByPoints(mygeo, mybox[21], mybox[23], mybox[9], mybox[11] )
			mygroups[a].append( face )
			
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# back1
			face = geompy.GetFaceByPoints(mygeo, mybox[1], mybox[2], mybox[24], mybox[25] )
			mygroups[a].append( face )
			face = geompy.GetFaceByPoints(mygeo, mybox[24], mybox[25], mybox[4], mybox[5] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# back2
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[22], mybox[25] )
			mygroups[a].append( face )
			face = geompy.GetFaceByPoints(mygeo, mybox[22], mybox[25], mybox[3], mybox[4] )
			mygroups[a].append( face )
			
		i = 0
		for b in mygroups:
			group = geompy.CreateGroup(mygeo, geompy.ShapeType["FACE"])
			m = False
			for a in b:
				id = geompy.GetSubShapeID(mygeo, a)
				geompy.AddObject(group, id)
				m = True
			if m:
				geompy.addToStudyInFather(mygeo, group, myfacenames[i])
				mygroups2.append(group)
			i = i + 1
	
	return mygroups2