"""     Box.py

The program SALOME is required to run this script. SALOME is available from https://www.salome-platform.org
Tested with SALOME 9.3.0 and 9.5.0.

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

#############################################################################################
### FUNCTION to define 8 corner points of a box
#############################################################################################
def DefPts(geompy, math, mx, my, mz, x0, y0, z0, rot, mybox):
	#: compute 8 points (8 corners) and add them to a list -> mybox (mybox must be an empty list !!) 
	# x-vector = mybox[0], mybox[1]
	# y-vector = mybox[0], mybox[3]
	# z-vector = mybox[0], mybox[6]
	myangle = 90 
	rot = math.radians(rot) # roation of the whole shape
	q = math.radians(myangle) # full range
	mydia = math.sqrt( mx * mx + my * my )	 # diagonal
	#h = math.acos( mx / mydia )
	h = math.asin( my / mydia )
	#h = math.radians( int(myangle / 2) ) # half range 
	# circular coordinates, rot should be either: 0, 90, 180, 270
	# x = x * math.cos(rot)
	# y = x * math.sin(rot)
	
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True
	mydisplayc = 0
	
	a = x0
	b = y0
	c = z0
	pt = geompy.MakeVertex ( a, b, c )	# 0
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + mx * math.cos(rot)
	b = y0 + mx * math.sin(rot)
	pt = geompy.MakeVertex ( a, b, c )	# 1
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + mydia * math.cos(rot + h)
	b = y0 + mydia * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )	 # 2
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + my * math.cos(rot + q)
	b = y0 + my * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )		# 3
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	c = z0 + mz
	
	a = x0 + mydia * math.cos(rot + h)
	b = y0 + mydia * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + my * math.cos(rot + q)
	b = y0 + my * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )		# 5
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0
	b = y0
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + mx * math.cos(rot)
	b = y0 + mx * math.sin(rot)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

############################################################################################
### FUNCTION to create the box 
#############################################################################################
def Make(geompy, mybox):
    # create edges, faces and make a SOLID from all faces	
    # this may be a bit complicated but it allows also for rotated box 
    # geompy.MakeBoxTwoPnt fails if box points are rotated by 45 degree (if both points are in the same plane)
	myedge = []
	myface = []
	
	# create edges
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True
	mydisplayc = 0
	
	edge = geompy.MakeEdge(mybox[0], mybox[1])
	myedge.append(edge)		# 0
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[1], mybox[2])
	myedge.append(edge)		# 1
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[2], mybox[3])
	myedge.append(edge)		# 2
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[0], mybox[3])
	myedge.append(edge)		# 3
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[0], mybox[6])
	myedge.append(edge)		# 4
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[1], mybox[7])
	myedge.append(edge)		# 5
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[2], mybox[4])
	myedge.append(edge)		# 6
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[3], mybox[5])
	myedge.append(edge)		# 7
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[6], mybox[7])
	myedge.append(edge)		# 8
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[4], mybox[7])
	myedge.append(edge)		# 9
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[4], mybox[5])
	myedge.append(edge)		# 10
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[5], mybox[6])
	myedge.append(edge)		# 11
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	# create faces
	
	wire = geompy.MakeWire( [myedge[0], myedge[1], myedge[2], myedge[3]] )	# first we make a WIRE 
	face = geompy.MakeFace(wire, True) # isPlanarWanted = True
	myface.append(face)		#  face 0
	
	wire = geompy.MakeWire( [myedge[0], myedge[4], myedge[5], myedge[8]] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 1
	
	wire = geompy.MakeWire( [myedge[1], myedge[5], myedge[6], myedge[9]] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 2
	
	wire = geompy.MakeWire( [myedge[2], myedge[6], myedge[7], myedge[10]] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 3
	
	wire = geompy.MakeWire( [myedge[3], myedge[4], myedge[7], myedge[11]] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 4
	
	wire = geompy.MakeWire( [myedge[8], myedge[9], myedge[10], myedge[11]] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 5
	
    # create a shell from a set of faces
	shell = geompy.MakeShell( myface )
	
	# create a solid, bounded by the given shell
	solid = geompy.MakeSolid( [shell] )
	
	return solid
    
#############################################################################################
### FUNCTION to make groups for Box faces 
#############################################################################################
def Groups(geompy, mybox, mygeo, mygroups, myfacelist, myfacenames, myvolname):
	#: Create GROUPS to be addressed later as boundary for OpenFOAM
	# VOLUME GROUP is assigned if myvolname is not empty
	# FACE GROUPS are assigend if myfacelist is not empty
	# boundary faces are only available if no other object is joined at the respective face
	# this is the order: BoxFaces = [ bottom, top, front, back, left, right ]
	mygroups2 = []
	
	if len(myvolname) > 0:
		group = geompy.CreateGroup(mygeo, geompy.ShapeType["SOLID"])
		id = geompy.GetSubShapeID(mygeo, mygeo)
		geompy.AddObject(group, id)
		geompy.addToStudyInFather(mygeo, group, myvolname)
		mygroups2.append(group)
	
	### FACE GROUPS
	if len(myfacelist) > 0:
		
		i = 0
		a = myfacelist[i]
		if a > (-1):			# if face is boundary (not linked to other face)
			# bottom
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[2], mybox[3] )
			mygroups[a].append( face )  # collect face into the list for: wall, inlet, outlet, ...
		
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# top
			face = geompy.GetFaceByPoints(mygeo, mybox[4], mybox[5], mybox[6], mybox[7] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# front
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[6], mybox[7] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# back
			face = geompy.GetFaceByPoints(mygeo, mybox[2], mybox[3], mybox[4], mybox[5] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# left
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[3], mybox[5], mybox[6] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# right
			face = geompy.GetFaceByPoints(mygeo, mybox[1], mybox[2], mybox[4], mybox[7] )
			mygroups[a].append( face )
			
		## create face groups from list
		
		i = 0
		for b in mygroups:
			group = geompy.CreateGroup(mygeo, geompy.ShapeType["FACE"])
			m = False
			for a in b:
				id = geompy.GetSubShapeID(mygeo, a)
				geompy.AddObject(group, id)
				m = True
			if m:
				geompy.addToStudyInFather(mygeo, group, myfacenames[i])
				mygroups2.append(group)
			i = i + 1
	
	return mygroups2
		
#############################################################################################
### FUNCTION to make MESH for Box 
#############################################################################################
def Mesh(geompy, mybox, mygeo, hexa, segwidth, segheight, seglen):
	#: Create the mesh for box
	hexa.Segment() # global setting
	minseg = 2  # minimum number of mesh points along one edge
	
	# x direction
	myseg = segwidth # channel mesh here
	myseg = max( minseg, myseg )
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[1])		
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	edge = geompy.GetEdge(mygeo, mybox[4], mybox[5])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	# z-direction 
	myseg = segheight
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[6])		
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	edge = geompy.GetEdge(mygeo, mybox[2], mybox[4])		
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	# y-direction
	myseg = seglen
	myseg = max( minseg, myseg )
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[3])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	edge = geompy.GetEdge(mygeo, mybox[4], mybox[7])
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	#hexa.Prism()  
	hexa.Quadrangle()
	hexa.Hexahedron()
	hexa.Compute()