"""     BoxGrad.py

The program SALOME is required to run this script. SALOME is available from https://www.salome-platform.org
Tested with SALOME 9.3.0 and 9.5.0.

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

#############################################################################################
### FUNCTION to define 8 corner points of a box
#############################################################################################
def DefPts(geompy, math, mx, my, mz, x0, y0, z0, rot, mybox):
	#: compute 8 points (8 corners) and add them to a list -> mybox (mybox must be an empty list !!) 
	# x-vector = mybox[0], mybox[1]
	# y-vector = mybox[0], mybox[3]
	# z-vector = mybox[0], mybox[6]
	myangle = 90 
	rot = math.radians(rot) # roation of the whole shape
	q = math.radians(myangle) # full range
	mydia = math.sqrt( mx * mx + my * my )	 # diagonal
	h = math.asin( my / mydia )
	# circular coordinates, rot should be either: 0, 90, 180, 270
	# x = x * math.cos(rot)
	# y = x * math.sin(rot)
	mydiax = math.sqrt( 0.5 * mx * 0.5 * mx + my * my )
	hx = math.acos( 0.5 * mx / mydiax )
	
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True # for visualization: add box to the geometry
	mydisplayc = 0
	
	a = x0
	b = y0
	c = z0
	pt = geompy.MakeVertex ( a, b, c )	# 0
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + mx * math.cos(rot)
	b = y0 + mx * math.sin(rot)
	pt = geompy.MakeVertex ( a, b, c )	# 1
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + mydia * math.cos(rot + h)
	b = y0 + mydia * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c ) # 2
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + my * math.cos(rot + q)
	b = y0 + my * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )	# 3
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	c = z0 + mz
	
	a = x0 + mydia * math.cos(rot + h)
	b = y0 + mydia * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + my * math.cos(rot + q)
	b = y0 + my * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )		# 5
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0
	b = y0
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + mx * math.cos(rot)
	b = y0 + mx * math.sin(rot)
	pt = geompy.MakeVertex ( a, b, c )	# 7
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	### points at half distance
	a = x0 + 0.5 * mx * math.cos(rot)
	b = y0 + 0.5 * mx * math.sin(rot)
	c = z0
	pt = geompy.MakeVertex ( a, b, c )	# 8
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	c = z0 + mz 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + mydiax * math.cos(rot + hx)
	b = y0 + mydiax * math.sin(rot + hx)
	c = z0
	pt = geompy.MakeVertex ( a, b, c )	# 10
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	c = z0 + mz
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0
	b = y0
	c = z0 + 0.5 * mz
	pt = geompy.MakeVertex ( a, b, c )	# 12
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + mx * math.cos(rot)
	b = y0 + mx * math.sin(rot)
	pt = geompy.MakeVertex ( a, b, c )	# 13
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + mydia * math.cos(rot + h)
	b = y0 + mydia * math.sin(rot + h)
	pt = geompy.MakeVertex ( a, b, c )	# 14
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + my * math.cos(rot + q)
	b = y0 + my * math.sin(rot + q)
	pt = geompy.MakeVertex ( a, b, c )		# 15
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1


############################################################################################
### FUNCTION to create the box 
#############################################################################################
def Make(geompy, mybox):
    # create edges, faces and make a SOLID from all faces	
    # this may be a bit complicated but it allows also for rotated box 
    # geompy.MakeBoxTwoPnt fails if box points are rotated by 45 degree (if both points are in the same plane)
	myedge = []
	myface = []
	
	# create edges
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True
	mydisplayc = 0
	
	edge = geompy.MakeEdge(mybox[0], mybox[8])
	myedge.append(edge)		# 0
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[8], mybox[1])
	myedge.append(edge)		# 1
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[1], mybox[13])
	myedge.append(edge)		# 2
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[13], mybox[7])
	myedge.append(edge)		# 3
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[7], mybox[9])
	myedge.append(edge)		# 4
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[9], mybox[6])
	myedge.append(edge)		# 5
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[6], mybox[12])
	myedge.append(edge)		# 6
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[12], mybox[0])
	myedge.append(edge)		# 7
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	
	
	edge = geompy.MakeEdge(mybox[3], mybox[10])
	myedge.append(edge)		# 8
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[10], mybox[2])
	myedge.append(edge)		# 9
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[2], mybox[14])
	myedge.append(edge)		# 10
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[14], mybox[4])
	myedge.append(edge)		# 11
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[4], mybox[11])
	myedge.append(edge)		# 12
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[11], mybox[5])
	myedge.append(edge)		# 13
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[5], mybox[15])
	myedge.append(edge)		# 14
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[15], mybox[3])
	myedge.append(edge)		# 15
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	
	edge = geompy.MakeEdge(mybox[0], mybox[3])
	myedge.append(edge)		# 16
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[1], mybox[2])
	myedge.append(edge)		# 17
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[5], mybox[6])
	myedge.append(edge)		# 18
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[4], mybox[7])
	myedge.append(edge)		# 19
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	# create faces
	
	wire = geompy.MakeWire( [ myedge[0], myedge[1], myedge[2], myedge[3], myedge[4], myedge[5], myedge[6], myedge[7] ] )	# first we make a WIRE 
	face = geompy.MakeFace(wire, True) # isPlanarWanted = True
	myface.append(face)		#  face 0
	
	wire = geompy.MakeWire( [ myedge[8], myedge[9], myedge[10], myedge[11], myedge[12], myedge[13], myedge[14], myedge[15] ] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 1
	
	wire = geompy.MakeWire( [ myedge[0], myedge[1], myedge[17], myedge[9], myedge[8], myedge[16] ] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 2
	
	wire = geompy.MakeWire( [ myedge[5], myedge[4], myedge[19], myedge[12], myedge[13], myedge[18] ] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 3
	
	wire = geompy.MakeWire( [ myedge[16], myedge[15], myedge[14], myedge[18], myedge[6], myedge[7] ] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 4
	
	wire = geompy.MakeWire( [ myedge[17], myedge[10], myedge[11], myedge[19], myedge[3], myedge[2] ] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 5
	
    # create a shell from a set of faces
	shell = geompy.MakeShell( myface )
	
	# create a solid, bounded by the given shell
	solid = geompy.MakeSolid( [shell] )
	
	return solid
    
#############################################################################################
### FUNCTION to make groups for Box faces 
#############################################################################################
def Groups(geompy, mybox, mygeo, mygroups, myfacelist, myfacenames, myvolname):
	#: Create GROUPS to be addressed later as boundary for OpenFOAM
	# VOLUME GROUP is assigned if myvolname is not empty
	# FACE GROUPS are assigend if myfacelist is not empty
	# boundary faces are only available if no other object is joined at the respective face
	# this is the order: BoxFaces = [ bottom, top, front, back, left, right ]
	mygroups2 = []
	
	if len(myvolname) > 0:
		group = geompy.CreateGroup(mygeo, geompy.ShapeType["SOLID"])
		id = geompy.GetSubShapeID(mygeo, mygeo)
		geompy.AddObject(group, id)
		geompy.addToStudyInFather(mygeo, group, myvolname)
		mygroups2.append(group)
	
	### FACE GROUPS
	if len(myfacelist) > 0:
		
		i = 0
		a = myfacelist[i]
		if a > (-1):			# if face is boundary (not linked to other face)
			# bottom
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[2], mybox[3] )
			mygroups[a].append( face )  # collect face into the list for: wall, inlet, outlet, ...
		
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# top
			face = geompy.GetFaceByPoints(mygeo, mybox[4], mybox[5], mybox[6], mybox[7] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# front
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[6], mybox[7] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# back
			face = geompy.GetFaceByPoints(mygeo, mybox[2], mybox[3], mybox[4], mybox[5] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# left
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[3], mybox[5], mybox[6] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if myfacelist[i] > (-1):
			# right
			face = geompy.GetFaceByPoints(mygeo, mybox[1], mybox[2], mybox[4], mybox[7] )
			mygroups[a].append( face )
			
		## create face groups from list
		
		i = 0
		for b in mygroups:
			group = geompy.CreateGroup(mygeo, geompy.ShapeType["FACE"])
			m = False
			for a in b:
				id = geompy.GetSubShapeID(mygeo, a)
				geompy.AddObject(group, id)
				m = True
			if m:
				geompy.addToStudyInFather(mygeo, group, myfacenames[i])
				mygroups2.append(group)
			i = i + 1
	
	return mygroups2
		
#############################################################################################
### FUNCTION to make MESH for Box 
#############################################################################################
def Mesh(geompy, mybox, mygeo, hexa, seglen, seggrad):
	#: Create the mesh for box
	algo1D = hexa.Segment()
	segmin = seggrad * 0.2
	segmax = seggrad * 0.8
	
	# x direction
	edge1 = geompy.GetEdge(mygeo, mybox[8], mybox[1])
	edge5 = geompy.GetEdge(mygeo, mybox[6], mybox[9])
	edge9 = geompy.GetEdge(mygeo, mybox[10], mybox[2])
	edge13 = geompy.GetEdge(mygeo, mybox[5], mybox[11])
	
	# z-direction 
	edge7 = geompy.GetEdge(mygeo, mybox[0], mybox[12])
	edge3 = geompy.GetEdge(mygeo, mybox[13], mybox[7])
	edge15 = geompy.GetEdge(mygeo, mybox[3], mybox[15])
	edge11 = geompy.GetEdge(mygeo, mybox[14], mybox[4])
	
	medges = [edge1, edge3, edge5, edge7, edge9, edge11, edge13, edge15]
	algo1D.Arithmetic1D( segmin, segmax, medges)
	algo1D.PropagationOfDistribution()
	
	# y-direction
	edge16 = geompy.GetEdge(mygeo, mybox[0], mybox[3])
	algo_local = hexa.Segment(edge16)
	algo_local.NumberOfSegments(seglen)
	algo_local.Propagation()
	
	#hexa.Prism()  
	hexa.Quadrangle()
	hexa.Hexahedron()
	hexa.Compute()
	