"""     Pie.py

The program SALOME is required to run this script. SALOME is available from https://www.salome-platform.org
Tested with SALOME 9.3.0 and 9.5.0.

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

#############################################################################################
### FUNCTION to define all points for pie
#############################################################################################
def DefPts(geompy, math, x, z, x0, y0, z0, myangle, rot, mybox):
	# compute all 20 points
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True
	mydisplayc = 0
	rot = math.radians(rot) # conversion to rad
	myangle = math.radians(myangle)  
	# circular coordinates, rot should be either: 0, 90, 180, 270
	# x = rad * math.cos(rot)
	# y = rad * math.sin(rot)
	
	## arc0 
	# inner point
	a = x0
	b = y0
	c = z0
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1

	a = x0 + x * math.cos(rot)
	b = y0 + x * math.sin(rot)
	c = z0 
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + x * math.cos(rot + 0.5 * myangle)
	b = y0 + x * math.sin(rot + 0.5 * myangle)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + x * math.cos(rot + myangle)
	b = y0 + x * math.sin(rot + myangle)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	## arc1 / copy in z direction
	# inner point
	a = x0
	b = y0
	c = z0 + z
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
		
	a = x0 + x * math.cos(rot)
	b = y0 + x * math.sin(rot)
	c = z0 + z
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + x * math.cos(rot + 0.5 * myangle)
	b = y0 + x * math.sin(rot + 0.5 * myangle)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
	a = x0 + x * math.cos(rot + myangle)
	b = y0 + x * math.sin(rot + myangle)
	pt = geompy.MakeVertex ( a, b, c )
	mybox.append(pt)
	if mydisplay:
		test = 'pt' + str(mydisplayc)
		geompy.addToStudy(pt, test)
		mydisplayc = mydisplayc + 1
	
############################################################################################
### FUNCTION to create the shape from a list of points
#############################################################################################
def Make(geompy, mybox):
	### make a solid piece of pie
	# create 2 arcs
	# create 10 edges
	# create all faces
	# make a SOLID from all faces	
	myarc = []
	myedge = []
	myface = []
	mydisplay = False # for visualization: add box to the geometry
	#mydisplay = True
	mydisplayc = 0
	
	#######################################################
	### create arcs from three points
	#######################################################
	arc = geompy.MakeArc(mybox[1], mybox[2], mybox[3])
	myarc.append(arc)
	if mydisplay:
		ptname = "myArc0"
		geompy.addToStudy(arc, ptname)						
	
	arc = geompy.MakeArc(mybox[5], mybox[6], mybox[7])
	myarc.append(arc)
	if mydisplay:
		ptname = "myArc1"
		geompy.addToStudy(arc, ptname)
	
	#######################################################
	### create edges
	#######################################################
	edge = geompy.MakeEdge(mybox[0], mybox[1])
	myedge.append(edge)		# 0
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[4], mybox[5])
	myedge.append(edge)		# 1
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[0], mybox[3])
	myedge.append(edge)		# 2
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[4], mybox[7])
	myedge.append(edge)		# 3
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[0], mybox[4])
	myedge.append(edge)		# 4
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	edge = geompy.MakeEdge(mybox[1], mybox[5])
	myedge.append(edge)		# 5
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
		
	edge = geompy.MakeEdge(mybox[3], mybox[7])
	myedge.append(edge)		# 6
	if mydisplay:
		test = 'edge ' + str(mydisplayc)
		geompy.addToStudy(edge, test)
		mydisplayc = mydisplayc + 1
	
	#######################################################
	### create faces
	#######################################################
	# first we make a WIRE 
	wire = geompy.MakeWire( [ myedge[0], myedge[1], myedge[4], myedge[5] ] )
	face = geompy.MakeFace(wire, True) # isPlanarWanted = True
	myface.append(face)		#  face 0
	
	wire = geompy.MakeWire( [ myedge[2], myedge[3], myedge[4], myedge[6] ] )
	face = geompy.MakeFace(wire, True) # isPlanarWanted = True
	myface.append(face)		#  face 1
	
	wire = geompy.MakeWire( [ myarc[0], myedge[0], myedge[2] ] )	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 2
	
	wire = geompy.MakeWire( [ myarc[1], myedge[1], myedge[3] ])	
	face = geompy.MakeFace(wire, True) 
	myface.append(face)		#  face 3
	
	face = geompy.MakePipe( myarc[0], myedge[4], 0 )	# we need to use MakePipe ( = extrusion of line to face) 
	myface.append(face)		#  face 4
	
	#######################################################
	### create SOLID
	#######################################################
	# create a shell from a set of faces
	shell = geompy.MakeShell( myface )
	
	# create a solid, bounded by the given shell
	solid = geompy.MakeSolid( [shell] )
	
	if mydisplay:
		ptname = "pie"
		geompy.addToStudy(solid, ptname)
		
	return solid

#############################################################################################
### FUNCTION to make groups for bend faces 
#############################################################################################
def Groups(geompy, mybox, mygeo, mygroups, myfacelist, myfacenames, myvolname):
	#: Create GROUPS to be addressed later as boundary for OpenFOAM
	# VOLUME GROUP is assigned if myvolname is not empty
	# FACE GROUPS are assigend if myfacelist is not empty
	#: This function gives only 3 faces: top, bottom, outer
	# myfacelist = [ 1, 2, 3, 4, 0, 0 ] # this assigns the different groups to the 6 sides of the box
	# a value of -1 means this face should NOT be assigned (e. g. inflow direction)
	# faces are only available if no other object is joined at the respective face
	# this is the order: # PieFaces = [bottom, top, outer, front, back]
	mygroups2 = []
	
	if len(myvolname) > 0:
		group = geompy.CreateGroup(mygeo, geompy.ShapeType["SOLID"])
		id = geompy.GetSubShapeID(mygeo, mygeo)
		geompy.AddObject(group, id)
		geompy.addToStudyInFather(mygeo, group, myvolname)
		mygroups2.append(group)
	
	### FACE GROUPS
	if len(myfacelist) > 0:
		i = 0
		a = myfacelist[i]
		if a > (-1):
			# bottom
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[0], mybox[3] )
			mygroups[a].append( face ) 
		
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# top
			face = geompy.GetFaceByPoints(mygeo, mybox[4], mybox[5], mybox[4], mybox[7] )
			mygroups[a].append( face ) 
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# outer
			face = geompy.GetFaceByPoints(mygeo, mybox[1], mybox[3], mybox[5], mybox[7] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# front
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[4], mybox[5] )
			mygroups[a].append( face )
			
		i = i + 1
		a = myfacelist[i]
		if a > (-1):
			# back
			face = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[3], mybox[4], mybox[7] )
			mygroups[a].append( face )
		
		i = 0
		for b in mygroups:
			group = geompy.CreateGroup(mygeo, geompy.ShapeType["FACE"])
			m = False
			for a in b:
				id = geompy.GetSubShapeID(mygeo, a)
				geompy.AddObject(group, id)
				m = True
			if m:
				geompy.addToStudyInFather(mygeo, group, myfacenames[i])
				mygroups2.append(group)
			i = i + 1
		
	return mygroups2

#############################################################################################
### FUNCTION to make prism (triangle) MESH for the shape 
#############################################################################################
def Mesh(geompy, mybox, mygeo, hexa, segwidth, segheight, segarc):
	minseg = 2  # minimum number of mesh points along one edge
	# define  edges
	arc = [ mybox[2], mybox[6] ]
	xside = [ mybox[0], mybox[1], mybox[0], mybox[3], mybox[4], mybox[5], mybox[4], mybox[7] ]

	# x-direction
	i = 0
	myseg = segwidth
	myseg = max( minseg, myseg )
	while i < 8:
		edge = geompy.GetEdge(mygeo, xside[i], xside[i+1])
		algo_local = hexa.Segment(edge)
		algo_local.NumberOfSegments(myseg)
		i = i + 2
		
	# z-direction 
	myseg = segheight
	myseg = max( minseg, myseg )
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[4])		
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge(mygeo, mybox[1], mybox[5])		
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	edge = geompy.GetEdge(mygeo, mybox[3], mybox[7])		
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	
	# arcs
	myseg = segarc
	myseg = max( minseg, myseg )
	for a in arc:
		edge = geompy.GetEdgeNearPoint(mygeo, a)
		algo_local = hexa.Segment(edge)
		algo_local.NumberOfSegments(myseg)

	# define vertex which is Triangle
	myVertex = geompy.GetVertexNearPoint(mygeo, mybox[4])	
	vertexID = geompy.GetSubShapeID( mygeo, myVertex )
	Quadrangle_2D = hexa.Quadrangle().TriangleVertex( vertexID )
	
	hexa.Prism()  
	hexa.Quadrangle()
	hexa.Hexahedron()
	hexa.Compute()		# compute mesh
    
#############################################################################################
### FUNCTION to make prism (triangle) MESH for the shape 
#############################################################################################
def MeshTri(geompy, mybox, mygeo, hexa, segwidth, segheight, segarc):
	# this function uses a very  simple mesh algorithm
	hexa.Prism()  
	hexa.Quadrangle()
	hexa.Hexahedron()
	
	minseg = 2  # minimum number of mesh points along one edge
	
	# define  edges
	arc = [ mybox[2], mybox[6] ]
	xside = [ mybox[0], mybox[1], mybox[0], mybox[3] ]

	# x-direction
	i = 0
	myseg = segwidth
	myseg = max( minseg, myseg )
	while i < 3:
		edge = geompy.GetEdge(mygeo, xside[i], xside[i+1])
		#edgelen = geompy.BasicProperties(edge)
		algo_local = hexa.Segment(edge)
		algo_local.NumberOfSegments(myseg)
		algo_local.Propagation()
		i = i + 2
		
	# z-direction 
	myseg = segheight
	#myseg = max( minseg, myseg )
	edge = geompy.GetEdge(mygeo, mybox[0], mybox[4])		
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	# arcs
	myseg = segarc
	myseg = max( minseg, myseg )
	for a in arc:
		edge = geompy.GetEdgeNearPoint(mygeo, a)
		algo_local = hexa.Segment(edge)
		algo_local.NumberOfSegments(myseg)
	
	# these are the faces of the pie
	#front = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[4], mybox[5] )
	#back = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[3], mybox[4], mybox[7] )
	bottom = geompy.GetFaceByPoints(mygeo, mybox[0], mybox[1], mybox[0], mybox[3] )
	#top = geompy.GetFaceByPoints(mygeo, mybox[4], mybox[5], mybox[4], mybox[7] )
	#outer = geompy.GetFaceByPoints(mygeo, mybox[1], mybox[3], mybox[5], mybox[7] )

	hexa.Triangle(bottom)
	algo2D = hexa.Triangle()
	#algo2D.LengthFromEdges()
	hexa.Compute()		# compute mesh