"""
###################################################
Subunit:  Coolant Layer / Anode 1

- Block 1 / 4 
with inlet region

Uwe Reimer, Forschungszentrum Juelich / Germany
February 2021
###################################################
"""

#############################################################################################
### FUNCTION to make geometry   -> with inlet / outlet region
#############################################################################################
def CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolname,  allgeolist, allmeshlist):
	#: Create the serpentine 
	# info:  if subshapes are not on display in the SALOME study, the face groups will not be copied to the mesh
	x0 = inputgeo[0]
	y0 = inputgeo[1]
	z0 = inputgeo[2]
	cw = inputgeo[3] 	# channel width 
	ch = inputgeo[4]	# channel height
		
	segx = inputmesh[0]
	segy = segx
	segz = inputmesh[1]
	
	mygroups = myfacegroups
	
	geolist = []	
	meshlist = []
	mybox = []
	obc = 0 # object counter
	mydisplayname = 'Anode1'
	
	## load basic geometry modules ##
	import Box
	
	#############################
	### START Inlet zone       ##
	#############################

	z0 = z0 + ch	# Anode liegt oben

	#############################
	### right piano channel
	#############################
	x0 = inputgeo[0]
	y0 = y0 + 4.0 * cw
	mylen = 56.0 * cw
	seglen = 56 * segx
	
	mybox = []
	Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)  
	myobject = Box.Make(geompy, mybox)
	test = mydisplayname + str(obc)
	geompy.addToStudy(myobject, test)
	geolist.append(myobject)
	# creategroups:	
	for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
		del a[0:]
	# BoxFaces = [ bottom, top, front, back, left, right ]
	faces = [-1, 0, -1, -1, 0, 0]
	grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
	# createmesh:
	hexa = smesh.Mesh(myobject) 
	Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
	for a in grouplist:		# copy groups from geometry
		hexa.GroupOnGeom(a)
	meshlist.append(hexa)			# collect submesh in list	
	
	obc = obc + 1
	
	
	#############################
	### Quer 1 
	#############################
	x0 = inputgeo[0] + cw * 2.0
	mylen = 2.0 * cw
	seglen = 2 * segx
	
	chan = 27 
	num = 0 #  counter for channels
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		if num == 1:
			faces = [-1, 0, -1, -1, 0, -1]
		elif num == chan:
			faces = [-1, 0, -1, -1, -1, 0]
		else:
			faces = [-1, 0, -1, -1, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	x0 = inputgeo[0] + cw * 2.0 + cw
	chan = 26
	num = 0 #  counter for channels
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [0, 0, 0, 0, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	
	#############################
	### 3 bottom channels
	#############################
	x0 = inputgeo[0] + cw * 2.0 * 28.0
	mylen = 56.0 * cw
	seglen = 56 * segx
	
	mybox = []
	Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
	myobject = Box.Make(geompy, mybox)
	test = mydisplayname + str(obc)
	geompy.addToStudy(myobject, test)
	geolist.append(myobject)
	# creategroups:	
	for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
		del a[0:]
	# BoxFaces = [ bottom, top, front, back, left, right ]
	faces = [-1, 0, -1, -1, 0, 0]
	grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
	# createmesh:
	hexa = smesh.Mesh(myobject) 
	Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
	for a in grouplist:		# copy groups from geometry
		hexa.GroupOnGeom(a)
	meshlist.append(hexa)			# collect submesh in list	
	obc = obc + 1
	
	x0 = x0 + cw * 2.0
	
	mybox = []
	Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
	myobject = Box.Make(geompy, mybox)
	test = mydisplayname + str(obc)
	geompy.addToStudy(myobject, test)
	geolist.append(myobject)
	# creategroups:	
	for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
		del a[0:]
	# BoxFaces = [ bottom, top, front, back, left, right ]
	faces = [-1, 0, -1, -1, 0, 0]
	grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
	# createmesh:
	hexa = smesh.Mesh(myobject) 
	Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
	for a in grouplist:		# copy groups from geometry
		hexa.GroupOnGeom(a)
	meshlist.append(hexa)			# collect submesh in list	
	
	obc = obc + 1
	x0 = x0 + cw * 2.0
		
	mybox = []
	Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
	myobject = Box.Make(geompy, mybox)
	test = mydisplayname + str(obc)
	geompy.addToStudy(myobject, test)
	geolist.append(myobject)
	# creategroups:	
	for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
		del a[0:]
	# BoxFaces = [ bottom, top, front, back, left, right ]
	faces = [-1, 0, -1, -1, 0, 0]
	grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
	# createmesh:
	hexa = smesh.Mesh(myobject) 
	Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
	for a in grouplist:		# copy groups from geometry
		hexa.GroupOnGeom(a)
	meshlist.append(hexa)			# collect submesh in list	
	
	obc = obc + 1
	x0 = x0 + cw * 2.0
	
	
	
	#############################
	### left piano channels
	#############################
	x0 = inputgeo[0] + cw * 2.0 * 31.0
	mylen = 56.0 * cw
	seglen = 56 * segx
	
	mybox = []
	Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
	myobject = Box.Make(geompy, mybox)
	test = mydisplayname + str(obc)
	geompy.addToStudy(myobject, test)
	geolist.append(myobject)
	# creategroups:	
	for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
		del a[0:]
	# BoxFaces = [ bottom, top, front, back, left, right ]
	faces = [-1, 0, -1, -1, 0, 0]
	grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
	# createmesh:
	hexa = smesh.Mesh(myobject) 
	Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
	for a in grouplist:		# copy groups from geometry
		hexa.GroupOnGeom(a)
	meshlist.append(hexa)			# collect submesh in list	
	
	obc = obc + 1
	x0 = x0 + cw * 2.0
	
	mybox = []
	Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
	myobject = Box.Make(geompy, mybox)
	test = mydisplayname + str(obc)
	geompy.addToStudy(myobject, test)
	geolist.append(myobject)
	# creategroups:	
	for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
		del a[0:]
	# BoxFaces = [ bottom, top, front, back, left, right ]
	faces = [-1, 0, -1, -1, 0, 0]
	grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
	# createmesh:
	hexa = smesh.Mesh(myobject) 
	Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
	for a in grouplist:		# copy groups from geometry
		hexa.GroupOnGeom(a)
	meshlist.append(hexa)			# collect submesh in list	
	
	obc = obc + 1
	x0 = x0 + cw * 2.0
	
	#############################
	### Block 1
	#############################
	x0 = inputgeo[0] + cw * 2.0
	y0 = inputgeo[1] + cw * 6.0
	mylen = 54.0 * cw
	seglen = 54 * segx
	
	chan = 27 
	num = 0 #  counter for channels
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, -1, -1, 0, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	################################################################################
	#############################
	### Start Triangle to add closing of left over spaces
	#############################
	mylen = 1.0 * cw
	seglen = 1 * segx
	
	chan0 = 19
	chan = chan0  
	while chan > 0:
		x0 = inputgeo[0] + cw * 24.0 + cw * ( chan0 - chan )
		y0 = inputgeo[1] + 12.0 * cw + cw * (chan0 - chan)
	
		#################################################################
		num = 0 
		while num < chan:
			num = num + 1
					
			mybox = []
			Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
			myobject = Box.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			# creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BoxFaces = [ bottom, top, front, back, left, right ]
			faces = [0, -1, -1, -1, -1, -1]
			grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			# createmesh:
			hexa = smesh.Mesh(myobject) 
			Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list	
			
			obc = obc + 1
			x0 = x0 + cw * 2.0
			
		chan = chan - 2
	
	#################################################################################
	### join all subshapes
	mygeo = geompy.MakePartition( geolist )	
	geompy.addToStudy(mygeo, mydisplayname)
	#geolist.append(mygeo)
	
	### now join all meshes
	MyCompound = smesh.Concatenate( meshlist, 1, True, 1e-05, False, name = mydisplayname)
	
	# garbage collection
	for a in meshlist:
		allmeshlist.append(a)
		
	for a in geolist:
		allgeolist.append(a)
		
	
	return MyCompound
#############################################################################################
### END of FUNCTIONS
#############################################################################################