"""
###################################################
Subunit:  Coolant Layer / Anode 5

Anode outlet cell manifold
ATTENTION: this is a copy of Anode 4 -> it is rotated at the end

Uwe Reimer, Forschungszentrum Juelich / Germany
March 2021
###################################################
"""

#############################################################################################
### FUNCTION to make geometry   -> with inlet / outlet region
#############################################################################################
def CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolname,  allgeolist, allmeshlist):
	#: Create the serpentine 
	# info:  if subshapes are not on display in the SALOME study, the face groups will not be copied to the mesh
	x0 = inputgeo[0]
	y0 = inputgeo[1]
	z0 = inputgeo[2]
	cw = inputgeo[3] 	# channel width 
	ch = inputgeo[4]	# channel height
		
	segx = inputmesh[0]
	segy = segx
	segz = inputmesh[1]

	mygroups = myfacegroups
	
	geolist = []	
	meshlist = []
	mybox = []
	obc = 0 # object counter
	mydisplayname = 'Anode'
	
	z0 = z0 + ch	# Anode liegt oben
	mylen1 = 1.0 * cw  # for single boxes
	seglen1 = 1 * segx
	
	## load basic geometry modules ##
	import Box, math
	
	#############################
	### START 4 chan right     ##
	#############################
	y0 = inputgeo[1] + 2.0 * cw
	mylen = 2.0 * cw
	seglen = 2 * segx
	
	chan = 4 
	num = 0 #  counter for channels
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, -1, -1, 0, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	y0 = inputgeo[1] 
	x0 = inputgeo[0]
	chan = 4 
	num = 0 #  counter for channels
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		if num == 1:
			faces = [0, 0, 0, -1, 0, -1]
		else:
			faces = [0, 0, 0, -1, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	###############  START insert box to close boundary  #######################
	x0 = x0 - cw * 2.0
	y0 = y0 + cw
	
	mybox = []
	Box.DefPts(geompy, math, cw, mylen1, ch, x0, y0, z0, 0, mybox)
	myobject = Box.Make(geompy, mybox)
	test = mydisplayname + str(obc)
	geompy.addToStudy(myobject, test)
	geolist.append(myobject)
	# creategroups:	
	for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
		del a[0:]
	# BoxFaces = [ bottom, top, front, back, left, right ]
	faces = [-1, -1, -1, -1, -1, 0]
	grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
	# createmesh:
	hexa = smesh.Mesh(myobject) 
	Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen1)	# compute mesh
	for a in grouplist:		# copy groups from geometry
		hexa.GroupOnGeom(a)
	meshlist.append(hexa)			# collect submesh in list		
	obc = obc + 1
	
	###############  END insert box to close boundary  #######################
	
	mylen = 2.0 * cw
	seglen = 2 * segx
	y0 = inputgeo[1] 
	x0 = inputgeo[0] + cw
	chan = 3 
	num = 0 
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [0, 0, 0, 0, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
		
	#############################
	### 10 chan aufsteigend    ##
	#############################
	
	mylen = 2.0 * cw 
	seglen = 2 * segx
	y0 = inputgeo[1] + cw * 2.0
	x0 = inputgeo[0] + cw * 8.0 
		
	chan = 10
	num = 0 
	while num < chan:
		num = num + 1
		
		mylen = mylen + cw
		seglen = seglen + segx
		y0 = y0 - cw		
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, -1, -1, 0, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	mylen = 2.0 * cw 
	seglen = 2 * segx
	y0 = inputgeo[1] 
	x0 = inputgeo[0] + cw * 7.0   # we start with "in between"
	
	chan = 10
	num = 0 
	while num < chan:
		num = num + 1
		
		y0 = y0 - cw		
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		#faces = [-1, 0, 0, 0, -1, -1]
		if num < 9:
			faces = [0, 0, 0, 0, -1, -1]
		else:
			faces = [-1, 0, 0, 0, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		
		###############  START insert box to close boundary  #######################
		mybox = []
		Box.DefPts(geompy, math, cw, mylen1, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, -1, -1, -1, 0, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen1)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list		
		obc = obc + 1
		###############  END insert box to close boundary  #######################
		
		x0 = x0 + cw 
		
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		#faces = [-1, 0, 0, -1, -1, -1]
		if num < 8:
			faces = [0, 0, 0, -1, -1, -1]
		else:
			faces = [-1, 0, 0, -1, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		
		###############  START insert box to close boundary  #######################
		y0 = y0 + cw
		
		mybox = []
		Box.DefPts(geompy, math, cw, mylen1, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, -1, -1, -1, -1, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen1)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list		
		obc = obc + 1
		
		y0 = y0 - cw
		###############  END insert box to close boundary  #######################
		
		x0 = x0 + cw 


	#############################
	### START 9 chan  middle   ##
	#############################
	x0 = inputgeo[0] + cw * 28.0
	y0 = inputgeo[1] - cw * (13.0 - 4.0)
	mylen = cw * 13.0
	seglen = segx * 13
	
	chan = 9 
	num = 0 #  counter for channels
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, -1, -1, 0, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	# quer chan
	x0 = inputgeo[0] + cw * 28.0
	y0 = inputgeo[1] - cw * (15.0 - 4.0)
	mylen = cw * 2.0
	seglen = segx * 2
	
	chan = 9 
	num = 0 #  counter for channels
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, 0, -1, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
		
	# quer in between
	x0 = inputgeo[0] + cw * 27.0
	mylen = cw * 2.0
	seglen = segx * 2
	
	chan = 10 
	num = 0 
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, -1, 0, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	###############  START insert boxes to close boundary  #######################
	x0 = inputgeo[0] + cw * 27.0
		
	chan = 10 
	num = 0 
	while num < chan:
		num = num + 1
		
		if (num < 3) or (num > 8) :
				
			mybox = []
			Box.DefPts(geompy, math, cw, mylen1, ch, x0, y0, z0, 0, mybox)
			myobject = Box.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			# creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BoxFaces = [ bottom, top, front, back, left, right ]
			if num == 1:
				faces = [-1, -1, 0, -1, 0, -1]
			elif num == chan:
				faces = [-1, -1, 0, -1, -1, 0]
			else:
				faces = [-1, -1, 0, -1, -1, -1]
			grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			# createmesh:
			hexa = smesh.Mesh(myobject) 
			Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen1)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list		
			obc = obc + 1
		
		x0 = x0 + cw * 2.0		
	###############  END insert box to close boundary  #######################
	
	
	#############################
	### 5 chan absteigend    ##
	#############################
	
	mylen = 12.0 * cw # one more
	seglen = 12 * segx
	x0 = inputgeo[0] + cw * 46.0
	y0 = inputgeo[1] - cw * (12.0 - 4.0)
		
	chan = 5
	num = 0 
	while num < chan:
		num = num + 1
		
		mylen = mylen - cw
		seglen = seglen - segx
		y0 = y0 + cw		
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, -1, -1, 0, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	mylen = 2.0 * cw 
	seglen = 2 * segx
	x0 = inputgeo[0] + cw * 46.0
	y0 = inputgeo[1] - cw * (14.0 - 4.0)
	
	chan = 5
	num = 0 
	while num < chan:
		num = num + 1
		
		y0 = y0 + cw		
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		if num == 1:
			faces = [-1, 0, -1, -1, 0, -1]
		elif num == chan:
			faces = [-1, 0, 0, -1, -1, 0]
		else:
			faces = [-1, 0, 0, -1, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		obc = obc + 1
		
		###############  START insert box to close boundary  #######################
		if num > 1:
			y0 = y0 + cw
			
			mybox = []
			Box.DefPts(geompy, math, cw, mylen1, ch, x0, y0, z0, 0, mybox)
			myobject = Box.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			# creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BoxFaces = [ bottom, top, front, back, left, right ]
			faces = [-1, -1, -1, -1, 0, -1]
			grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			# createmesh:
			hexa = smesh.Mesh(myobject) 
			Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen1)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list		
			obc = obc + 1
			
			y0 = y0 - cw
		###############  END insert box to close boundary  #######################
				
		x0 = x0 + cw * 2.0

	# in between
	x0 = inputgeo[0] + cw * 47.0
	y0 = inputgeo[1] - cw * (14.0 - 4.0)
	
	chan = 4
	num = 0 
	while num < chan:
		num = num + 1
		
		y0 = y0 + cw		
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, 0, 0, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		obc = obc + 1
		
		mybox = []		# one box to close side
		Box.DefPts(geompy, math, cw, mylen1, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, -1, -1, -1, -1, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen1)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		obc = obc + 1
		
		x0 = x0 + cw * 2.0

		
	### special corner
	x0 = inputgeo[0] + cw * 46.0
	y0 = inputgeo[1] - cw * (14.0 - 4.0)

	mybox = []
	Box.DefPts(geompy, math, cw, mylen1, ch, x0, y0, z0, 0, mybox)
	myobject = Box.Make(geompy, mybox)
	test = mydisplayname + str(obc)
	geompy.addToStudy(myobject, test)
	geolist.append(myobject)
	# creategroups:	
	for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
		del a[0:]
	# BoxFaces = [ bottom, top, front, back, left, right ]
	faces = [-1, 0, 0, -1, -1, 0]
	grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
	# createmesh:
	hexa = smesh.Mesh(myobject) 
	Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen1)	# compute mesh
	for a in grouplist:		# copy groups from geometry
		hexa.GroupOnGeom(a)
	meshlist.append(hexa)			# collect submesh in list	
	obc = obc + 1


	########################################
	### 5 chan from cathode (left side)   ##
	########################################
	
	mylen = 6.0 * cw 
	seglen = 6 * segx
	x0 = inputgeo[0] + cw * 56.0
	y0 = inputgeo[1] - cw * (6.0 - 4.0)
		
	chan = 5
	num = 0 
	while num < chan:
		num = num + 1
		
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [-1, 0, 0, -1, 0, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	#############################
	### OUTLET half height     ###
	#############################
	
	mylen = 6.0 * cw 
	seglen = 6 * segx
	x0 = inputgeo[0] + cw * 31.0
	y0 = inputgeo[1] - cw * (21.0 - 4.0)
		
	chan = 6
	num = 0 
	while num < chan:
		num = num + 1
		
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [0, 0, 3, -1, 0, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	
	
	#################################################################################
	### join all subshapes
	mygeo = geompy.MakePartition( geolist )	
	geompy.addToStudy(mygeo, mydisplayname)
	#geolist.append(mygeo)
	
	### now join all meshes
	MyCompound = smesh.Concatenate( meshlist, 1, True, 1e-05, False, name = mydisplayname)
	
	p1 = geompy.MakeVertex(0, 0, 0)
	p2 = geompy.MakeVertex(0, 0, 5.0)
	v = geompy.MakeVector(p1, p2)
	MyCompound.Rotate( [], v, math.pi, False, False)
	
	a = cw * 65.0
	p2 = geompy.MakeVertex(a, 0, 0)
	v = geompy.MakeVector(p1, p2)
	MyCompound.Translate( [], v, False, False)
	
	a = cw * 234.0
	p2 = geompy.MakeVertex(0, a, 0)
	v = geompy.MakeVector(p1, p2)
	MyCompound.Translate( [], v, False, False)
	
	# garbage collection
	for a in meshlist:
		allmeshlist.append(a)
		
	for a in geolist:
		allgeolist.append(a)
		
	
	return MyCompound
#############################################################################################
### END of FUNCTIONS
#############################################################################################