"""
###################################################
Subunit:  Coolant Layer / Kathode 1

- Block 2 + 3 / 4 
with inlet region

Uwe Reimer, Forschungszentrum Juelich / Germany
February 2021
###################################################
"""

#############################################################################################
### FUNCTION to make geometry   -> with inlet / outlet region
#############################################################################################
def CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolname,  allgeolist, allmeshlist):
	#: Create the serpentine 
	# info:  if subshapes are not on display in the SALOME study, the face groups will not be copied to the mesh
	x0 = inputgeo[0]
	y0 = inputgeo[1]
	z0 = inputgeo[2]
	cw = inputgeo[3] 	# channel width 
	ch = inputgeo[4]	# channel height
	
	segx = inputmesh[0]
	segy = segx
	segz = inputmesh[1]
	
	mygroups = myfacegroups
	
	geolist = []	
	meshlist = []
	mybox = []
	obc = 0 # object counter
	mydisplayname = 'Cathode2'
	
	## load basic geometry modules ##
	import Box
	
	y0 = inputgeo[1] + cw * 60.0
	
	mylen = (56.0 + 56.0) * cw
	seglen = (56 + 56) * segx
	
	chan = 33
	num = 0 #  counter for channels
	while num < chan:
		num = num + 1
				
		mybox = []
		Box.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		faces = [0, -1, -1, -1, 0, 0]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + cw * 2.0
	
	
	#################################################################################
	### join all subshapes
	mygeo = geompy.MakePartition( geolist )	
	geompy.addToStudy(mygeo, mydisplayname)
	#geolist.append(mygeo)
	
	### now join all meshes
	MyCompound = smesh.Concatenate( meshlist, 1, True, 1e-05, False, name = mydisplayname)
	
	# garbage collection
	for a in meshlist:
		allmeshlist.append(a)
		
	for a in geolist:
		allgeolist.append(a)
		
	
	return MyCompound
#############################################################################################
### END of FUNCTIONS
#############################################################################################