"""   Subunit:  Pillar2

 - creates channels for a separate serpentine flow field
 - additional length of channel for inlet / outlet is possible
 - basic shapes (objects) are loaded from separate files
 - intermediate shapes are removed from the study
   (see end of this file, can be commented out in order to display all subshapes for debugging)
 - assigns face groups and one volume group
 - requires: Box.py and Bend.py

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""
#############################################################################################
### FUNCTION to make geometry   -> with inlet / outlet region
#############################################################################################
def CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolname,  allgeolist, allmeshlist, tolerance):
    #: Create the mesh
    x0 = inputgeo[0]  
    y0 = inputgeo[1] 
    z0 = inputgeo[2]
    cw = inputgeo[3]    # channel width
    ch = inputgeo[6]    # GDL height
    ns = int( inputgeo[9])   # number of spacers
    myturns = int( inputgeo[7]) # half of number of U-turns
    ncol = int( inputgeo[8])   # number of colums
    inletn = int( inputgeo[10])   # number of spacers for inlet/ outlet
    
    myrib = cw / inputgeo[5]
    inrad = cw * 0.5 # half of channel width
    obc = 0     # object counter
    mydisplayname = 'Snake '
    geolist = []	
    meshlist = []
    
    outrad = inrad + cw 			# outer radius and
    bendrecsize = outrad + inrad	# size of Bendrec unit
    myangle = 90        # angle for Bend and BendRec 
    myres = float( inputmesh[0] / cw)	# mesh resolution
    #seggrad = float( inputgeo[4] / inputmesh[0] )
    segback = int( bendrecsize * myres )
    segback = segback * 2
    segarc = 2 * segback
    seglen = int( myres * cw ) * 2 # segments for spacer (x-direction)
    #segx = int( inputmesh[0] ) * 2
    #segz = int( inputmesh[3] ) # * 2  half the number, because height is two edges
    segwidth = inputmesh[0] * 2
    segheight = inputmesh[1] * 2
    
    ## load basic geometry modules ##
    import Box, Bend   
    
    
    ### loop over colums
    countcol = 0
    while countcol < ncol:
        countcol = countcol + 1

        ### one column
        
        nturn = 0
        while nturn < myturns:          
            nturn = nturn + 1
        
            # box upper spacer
            rot = 90
            x0 = x0 + float(cw * ns)
            
            mybox = [] 
            Box.DefPts(geompy, math, cw, float(cw * ns) , ch, x0, y0, z0, rot, mybox)
            myobject = Box.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
            # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BoxFaces = [ bottom, top, front, back, left, right ]
            faces = [-1, 1, -1, -1, -1, -1]
            grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            #createmesh:
            hexa = smesh.Mesh(myobject) 
            Box.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, (seglen * ns) )	# compute mesh
            for a in grouplist:		# copy groups from geometry
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            y0 = y0 + outrad
            
            mybox = []   
            rot = 270 # rotation in space
            Bend.DefPts(geompy, math, outrad, ch, inrad, x0, y0, z0, myangle, rot, mybox)	
            myobject = Bend.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            faces = [-1, 1, -1, -1, -1, -1]
            grouplist = Bend.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            Bend.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, segarc)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = []   # empty array to collect points in space
            rot = 0 # rotation in space
            Bend.DefPts(geompy, math, outrad, ch, inrad, x0, y0, z0, myangle, rot, mybox)	
            myobject = Bend.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            faces = [ -1, 1, -1, -1, -1, -1]
            grouplist = Bend.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            Bend.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, segarc)

            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)				
            obc = obc + 1
            
            # box lower spacer
            
            y0 = y0 + inrad
            rot = 90
            
            mybox = [] 
            Box.DefPts(geompy, math, cw, float(cw * ns) , ch, x0, y0, z0, rot, mybox)
            myobject = Box.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
            # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BoxFaces = [ bottom, top, front, back, left, right ]
            faces = [-1, 1, -1, -1, -1, -1]
            grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            #createmesh:
            hexa = smesh.Mesh(myobject) 
            Box.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, (seglen * ns))	# compute mesh
            for a in grouplist:		# copy groups from geometry
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            x0 = x0 - float(cw * ns)
            y0 = y0 + outrad
            
            rot = 180
            mybox = []   # empty array to collect points in space
            Bend.DefPts(geompy, math, outrad, ch, inrad, x0, y0, z0, myangle, rot, mybox)	
            myobject = Bend.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            faces = [-1, 1, -1, -1, -1, -1]
            grouplist = Bend.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            Bend.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, segarc)

            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            rot = 90  
            mybox = []   # empty array to collect points in space
            Bend.DefPts(geompy, math, outrad, ch, inrad, x0, y0, z0, myangle, rot, mybox)	
            myobject = Bend.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            faces = [-1, 1, -1, -1, -1, -1]
            grouplist = Bend.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            Bend.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, segarc)

            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            y0 = y0 + inrad
        
        # box outlet
        rot = 90
        x0 = x0 + float(cw * ns)
        
        mybox = [] 
        Box.DefPts(geompy, math, cw, float(cw * ns) , ch, x0, y0, z0, rot, mybox)
        myobject = Box.Make(geompy, mybox)
        test = mydisplayname + str(obc)
        geompy.addToStudy(myobject, test)
        geolist.append(myobject)
        # creategroups:	
        for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
            del a[0:]
        # BoxFaces = [ bottom, top, front, back, left, right ]
        faces = [-1, 1, -1, -1, -1, -1]
        grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
        #createmesh:
        hexa = smesh.Mesh(myobject) 
        Box.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, (seglen * ns) )	# compute mesh
        for a in grouplist:		# copy groups from geometry
            hexa.GroupOnGeom(a)
        meshlist.append(hexa)			# collect submesh in list	
        obc = obc + 1
        
        ### for the next colum
        x0 = inputgeo[0] + ( cw * 4.0 + cw * ns ) * countcol
        y0 = inputgeo[1] - 2.0 * cw * countcol
         
        
    ### loop over columns / inlet and outlet
    
    ## box inlet
    
    y0 = inputgeo[1]
    countcol = 0
    while countcol < ncol:
        countcol = countcol + 1
        rot = 90
       
        myseg = ( 2 + (countcol - 1) * (ns + 4) )
        mylen =  float( myseg ) * cw
        myseg = myseg * seglen
        
        x0 = inputgeo[0] + mylen - cw * 2.0
        
        mybox = [] 
        Box.DefPts(geompy, math, cw, mylen , ch, x0, y0, z0, rot, mybox)
        myobject = Box.Make(geompy, mybox)
        test = mydisplayname + str(obc)
        geompy.addToStudy(myobject, test)
        geolist.append(myobject)
        # creategroups:	
        for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
            del a[0:]
        # BoxFaces = [ bottom, top, front, back, left, right ]
        faces = [-1, 1, -1, 4, -1, -1]
        grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
        #createmesh:
        hexa = smesh.Mesh(myobject) 
        Box.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, myseg)	# compute mesh
        for a in grouplist:		# copy groups from geometry
            hexa.GroupOnGeom(a)
        meshlist.append(hexa)			# collect submesh in list	
        obc = obc + 1
            
        y0 = inputgeo[1] - 2.0 * cw * countcol
    
    ## box outlet
    
    y0 = inputgeo[1] + float( cw * myturns * 4.0 )
    
    mylen =  float( (ncol -1) * (ns + 4) + ns + 2 ) * cw
    x0 = inputgeo[0] + mylen 
    rot = 90
    
    countcol = 0
    while countcol < ncol:
        countcol = countcol + 1
        
        myseg = (ncol - countcol) * (ns + 4) + 2
        mylen =  float( myseg ) * cw
        myseg = myseg * seglen
        
        mybox = [] 
        Box.DefPts(geompy, math, cw, mylen , ch, x0, y0, z0, rot, mybox)
        myobject = Box.Make(geompy, mybox)
        test = mydisplayname + str(obc)
        geompy.addToStudy(myobject, test)
        geolist.append(myobject)
        # creategroups:	
        for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
            del a[0:]
        # BoxFaces = [ bottom, top, front, back, left, right ]
        faces = [-1, 1, 4, -1, -1, -1]        
        grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
        #createmesh:
        hexa = smesh.Mesh(myobject) 
        Box.Mesh(geompy, mybox, myobject, hexa, segwidth, segheight, myseg)	# compute mesh
        for a in grouplist:		# copy groups from geometry
            hexa.GroupOnGeom(a)
        meshlist.append(hexa)			# collect submesh in list	
        obc = obc + 1
        
        y0 = y0 - cw * 2.0 
    
    ### join all subshapes
    mygeo = geompy.MakePartition( geolist )	
    geompy.addToStudy(mygeo, 'Channels 2')
    
    ### now join all meshes
    MyCompound = smesh.Concatenate( meshlist, True, True, tolerance, False, name='Channels 2')
    #                              meshlist, unite groups, merge nodes, tolerance, force groups creation, name
    
    # prepare for cleaning up the study
    for a in meshlist:
        allmeshlist.append(a)
        
    for a in geolist:
        allgeolist.append(a)

    return MyCompound