"""   Subunit:  PillarRib2tri with tedrahedrons in half circle segments

 - creates ribs for a separate serpentine flow field
 - additional length of channel for inlet / outlet is possible
 - basic shapes (objects) are loaded from separate files
 - intermediate shapes are removed from the study
   (see end of this file, can be commented out in order to display all subshapes for debugging)
 - assigns face groups and one volume group
 - requires: Box.py  BendRec.py  Pie.py

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

#############################################################################################
### FUNCTION to make geometry   -> with inlet / outlet region
#############################################################################################
def CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolname,  allgeolist, allmeshlist, tolerance):
    #: Create the mesh
    x0 = inputgeo[0]  
    y0 = inputgeo[1] 
    z0 = inputgeo[2]
    cw = inputgeo[3]    # channel width
    cw2 = cw * 0.5
    ch = inputgeo[6]    # GDL height
    ns = int( inputgeo[9])   # number of spacers
    myturns = int( inputgeo[7]) # half of number of U-turns
    ncol = int( inputgeo[8])   # number of colums
    inletn = int( inputgeo[10])   # number of spacers for inlet/ outlet
    
    myrib = cw / inputgeo[5]
    inrad = cw * 0.5 # half of channel width
    obc = 0     # object counter
    mydisplayname = 'Snake '
    geolist = []	
    meshlist = []
    
    outrad = inrad + cw 			# outer radius and
    bendrecsize = outrad + inrad	# size of Bendrec unit
    myangle = 90        # angle for Bend and BendRec 
    myres = float( inputmesh[0] / cw)	# mesh resolution
    #seggrad = float( inputgeo[4] / inputmesh[0] )
    segback = int( bendrecsize * myres)
    segback = segback * 2
    segarc = 2 * segback
    # seglen = int( myres * cw ) # segments for spacer (x-direction)
    seglen = int( myres * cw )* 2 # segments for spacer (x-direction)
    segx = int( inputmesh[0] ) * 2
    segz = int( inputmesh[1] ) * 2
    
    ## load basic geometry modules ##
    import Box, BendRec, Pie
    
    ### loop over colums
    countcol = 0
    while countcol < ncol:
        countcol = countcol + 1

        ### one column
        y0 = y0 - cw2
        nturn = 0
        while nturn < myturns:          
            nturn = nturn + 1

            # box upper spacers
            
            rot = 90
            x0 = x0 + float(cw * ns)
            
            mybox = [] 
            Box.DefPts(geompy, math, cw2, float(cw * ns) , ch, x0, y0, z0, rot, mybox)
            myobject = Box.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
            # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BoxFaces = [ bottom, top, front, back, left, right ]
            if countcol == ncol and nturn == 1:
                faces = [4, 1, -1, -1, 4, -1]
            else:
                faces = [4, 1, -1, -1, -1, -1]
            grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            #createmesh:
            hexa = smesh.Mesh(myobject) 
            Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, (seglen * ns) )	# compute mesh
            for a in grouplist:		# copy groups from geometry
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = [] 
            Box.DefPts(geompy, math, cw2, float(cw * ns) , ch, x0, (y0 + cw + cw2), z0, rot, mybox)
            myobject = Box.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
            # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BoxFaces = [ bottom, top, front, back, left, right ]
            faces = [4, 1, -1, -1, -1, -1]
            grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            #createmesh:
            hexa = smesh.Mesh(myobject) 
            Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, (seglen * ns) )	# compute mesh
            for a in grouplist:		# copy groups from geometry
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            
            y0 = y0 + outrad + cw2
                    
            mybox = []   
            rot = 270 # rotation in space
            BendRec.DefPts(geompy, math, bendrecsize, ch, outrad, x0, y0, z0, rot, mybox)	
            myobject = BendRec.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendRecFaces = [ bottom, top, left, right, inner, back1, back2 ]
            if countcol == ncol:
                if nturn == 1:
                    faces = [4, 1, -1, -1, -1, 4, 4]
                else:
                    faces = [4, 1, -1, -1, -1, 4, -1]
            else:
                faces = [4, 1, -1, -1, -1, -1, -1]
            grouplist = BendRec.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            BendRec.Mesh(geompy, mybox, myobject, hexa, segx, segz, segarc, segback)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = []   
            rot = 0 # rotation in space
            BendRec.DefPts(geompy, math, bendrecsize, ch, outrad, x0, y0, z0, rot, mybox)	
            myobject = BendRec.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            if countcol == ncol:
                faces = [4, 1, -1, -1, -1, -1, 4]
            else:
                faces = [4, 1, -1, -1, -1, -1, -1]
            grouplist = BendRec.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            BendRec.Mesh(geompy, mybox, myobject, hexa, segx, segz, segarc, segback)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = []   
            rot = 270 # rotation in space
            Pie.DefPts(geompy, math, cw2, ch, x0, y0, z0, myangle, rot, mybox)
            myobject = Pie.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            faces = [4, 1, -1, -1, -1]
            grouplist = Pie.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            Pie.MeshTri(geompy, mybox, myobject, hexa, segx, segz, segarc)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = []   
            rot = 0 # rotation in space
            Pie.DefPts(geompy, math, cw2, ch, x0, y0, z0, myangle, rot, mybox)
            myobject = Pie.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            faces = [4, 1, -1, -1, -1]
            grouplist = Pie.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            Pie.MeshTri(geompy, mybox, myobject, hexa, segx, segz, segarc)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            x0 = x0 - float(cw * ns)
            
            # box lower spacers

            rot = 90
            x0 = x0 + float(cw * ns)
            
            mybox = [] 
            Box.DefPts(geompy, math, cw2, float(cw * ns) , ch, x0, y0, z0, rot, mybox)
            myobject = Box.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
            # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BoxFaces = [ bottom, top, front, back, left, right ]
            faces = [4, 1, -1, -1, -1, -1]
            grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            #createmesh:
            hexa = smesh.Mesh(myobject) 
            Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, (seglen * ns) )	# compute mesh
            for a in grouplist:		# copy groups from geometry
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = [] 
            Box.DefPts(geompy, math, cw2, float(cw * ns) , ch, x0, (y0 + cw + cw2), z0, rot, mybox)
            myobject = Box.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
            # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BoxFaces = [ bottom, top, front, back, left, right ]
            faces = [4, 1, -1, -1, -1, -1]
            grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            #createmesh:
            hexa = smesh.Mesh(myobject) 
            Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, (seglen * ns) )	# compute mesh
            for a in grouplist:		# copy groups from geometry
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            
            y0 = y0 +  cw * 2.0
            x0 = x0 - float( ns * cw)
            
            # box lower spacers - once again !!!!
            
            rot = 90
            x0 = x0 + float(cw * ns)
            
            mybox = [] 
            Box.DefPts(geompy, math, cw2, float(cw * ns), ch, x0, y0, z0, rot, mybox)
            myobject = Box.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
            # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BoxFaces = [ bottom, top, front, back, left, right ]
            faces = [4, 1, -1, -1, -1, -1]
            grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            #createmesh:
            hexa = smesh.Mesh(myobject) 
            Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, (seglen * ns) )	# compute mesh
            for a in grouplist:		# copy groups from geometry
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = [] 
            Box.DefPts(geompy, math, cw2, float(cw * ns) , ch, x0, (y0 + cw + cw2), z0, rot, mybox)
            myobject = Box.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
            # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BoxFaces = [ bottom, top, front, back, left, right ]
            if countcol == 1 and nturn == myturns:
                faces = [4, 1, -1, -1, -1, 4]
            else :
                faces = [4, 1, -1, -1, -1, -1]
            grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            #createmesh:
            hexa = smesh.Mesh(myobject) 
            Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, (seglen * ns) )	# compute mesh
            for a in grouplist:		# copy groups from geometry
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            x0 = x0 - float( ns * cw)
            
            mybox = []   
            rot = 180 # rotation in space
            BendRec.DefPts(geompy, math, bendrecsize, ch, outrad, x0, y0, z0, rot, mybox)	
            myobject = BendRec.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            if countcol == 1:
                faces = [4, 1, -1, -1, -1, -1, 4]
            else:
                faces = [4, 1, -1, -1, -1, -1, -1]
            grouplist = BendRec.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            BendRec.Mesh(geompy, mybox, myobject, hexa, segx, segz, segarc, segback)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = []   
            rot = 90 # rotation in space
            BendRec.DefPts(geompy, math, bendrecsize, ch, outrad, x0, y0, z0, rot, mybox)	
            myobject = BendRec.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            if countcol == 1:
                if nturn == myturns:
                    faces = [4, 1, -1, -1, -1, 4, 4]
                else:
                    faces = [4, 1, -1, -1, -1, 4, -1]
            else:
                faces = [4, 1, -1, -1, -1, -1, -1]
            grouplist = BendRec.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            BendRec.Mesh(geompy, mybox, myobject, hexa, segx, segz, segarc, segback)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = []   
            rot = 180 # rotation in space
            Pie.DefPts(geompy, math, cw2, ch, x0, y0, z0, myangle, rot, mybox)
            myobject = Pie.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            faces = [4, 1, -1, -1, -1]
            grouplist = Pie.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            Pie.MeshTri(geompy, mybox, myobject, hexa, segx, segz, segarc)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
            mybox = []   
            rot = 90 # rotation in space
            Pie.DefPts(geompy, math, cw2, ch, x0, y0, z0, myangle, rot, mybox)
            myobject = Pie.Make(geompy, mybox)
            test = mydisplayname + str(obc)
            geompy.addToStudy(myobject, test)
            geolist.append(myobject)
             # creategroups:	
            for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
                del a[0:]
            # BendFaces = [ bottom, top, outer, inner, front, back ]
            faces = [4, 1, -1, -1, -1]
            grouplist = Pie.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
            # create mesh
            hexa = smesh.Mesh(myobject) 
            Pie.MeshTri(geompy, mybox, myobject, hexa, segx, segz, segarc)
            for a in grouplist:		
                hexa.GroupOnGeom(a)
            meshlist.append(hexa)			# collect submesh in list	
            obc = obc + 1
            
                
        ### for the next colum
        x0 = inputgeo[0] + float( ( cw * 4.0 + cw * ns ) * countcol )
        y0 = inputgeo[1] - float( 2.0 * cw * countcol )
    
    
    
    
    ### loop over columns / inlet and outlet
    
    ## box inlet
    
    y0 = inputgeo[1] - cw2
    countcol = 0
    while countcol < ncol:
        countcol = countcol + 1
        rot = 90
       
        myseg = ( 2 + (countcol - 1) * (ns + 4) )
        mylen =  float( myseg ) * cw
        myseg = myseg * seglen
        
        x0 = inputgeo[0] + mylen - cw * 2.0
        
        mybox = [] 
        Box.DefPts(geompy, math, cw2, mylen, ch, x0, y0, z0, rot, mybox)
        myobject = Box.Make(geompy, mybox)
        test = mydisplayname + str(obc)
        geompy.addToStudy(myobject, test)
        geolist.append(myobject)
        # creategroups:	
        for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
            del a[0:]
        # BoxFaces = [ bottom, top, front, back, left, right ]
        if countcol == ncol:
            faces = [4, 1, -1, 4, 4, -1]
        else :
            faces = [4, 1, -1, 4, -1, -1]
        grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
        #createmesh:
        hexa = smesh.Mesh(myobject) 
        Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, myseg)	# compute mesh
        for a in grouplist:		# copy groups from geometry
            hexa.GroupOnGeom(a)
        meshlist.append(hexa)			# collect submesh in list	
        obc = obc + 1
        
        mybox = [] 
        Box.DefPts(geompy, math, cw2, mylen, ch, x0, (y0 + cw + cw2), z0, rot, mybox)
        myobject = Box.Make(geompy, mybox)
        test = mydisplayname + str(obc)
        geompy.addToStudy(myobject, test)
        geolist.append(myobject)
        # creategroups:	
        for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
            del a[0:]
        # BoxFaces = [ bottom, top, front, back, left, right ]
        faces = [4, 1, -1, 4, -1, -1]
        grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
        #createmesh:
        hexa = smesh.Mesh(myobject) 
        Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, myseg)	# compute mesh
        for a in grouplist:		# copy groups from geometry
            hexa.GroupOnGeom(a)
        meshlist.append(hexa)			# collect submesh in list	
        obc = obc + 1
            
        y0 = inputgeo[1] - 2.0 * cw * countcol - cw2
    

    ## box outlet
    
    
    y0 = inputgeo[1] + float( cw * myturns * 4.0 ) - cw2
    
    mylen =  float( (ncol -1) * (ns + 4) + ns + 2 ) * cw
    x0 = inputgeo[0] + mylen 
    rot = 90
    
    countcol = 0
    while countcol < ncol:
        countcol = countcol + 1
        
        myseg = (ncol - countcol) * (ns + 4) + 2
        mylen =  float( myseg ) * cw
        myseg = myseg * seglen
        
        mybox = [] 
        Box.DefPts(geompy, math, cw2, mylen , ch, x0, y0, z0, rot, mybox)
        myobject = Box.Make(geompy, mybox)
        test = mydisplayname + str(obc)
        geompy.addToStudy(myobject, test)
        geolist.append(myobject)
        # creategroups:	
        for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
            del a[0:]
        # BoxFaces = [ bottom, top, front, back, left, right ]
        faces = [4, 1, 4, -1, -1, -1]
        grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
        #createmesh:
        hexa = smesh.Mesh(myobject) 
        Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, myseg)	# compute mesh
        for a in grouplist:		# copy groups from geometry
            hexa.GroupOnGeom(a)
        meshlist.append(hexa)			# collect submesh in list	
        obc = obc + 1
        
        mybox = [] 
        Box.DefPts(geompy, math, cw2, mylen , ch, x0, (y0 + cw + cw2), z0, rot, mybox)
        myobject = Box.Make(geompy, mybox)
        test = mydisplayname + str(obc)
        geompy.addToStudy(myobject, test)
        geolist.append(myobject)
        # creategroups:	
        for a in myfacegroups:		# lists must be deleted, because PYTHON just 'links' the name
            del a[0:]
        # BoxFaces = [ bottom, top, front, back, left, right ]
        if countcol == 1 :
            faces = [4, 1, 4, -1, -1, 4]
        else :
            faces = [4, 1, 4, -1, -1, -1]
        grouplist = Box.Groups(geompy, mybox, myobject, myfacegroups, faces, myfacenames, myvolname)
        #createmesh:
        hexa = smesh.Mesh(myobject) 
        Box.Mesh(geompy, mybox, myobject, hexa, segx, segz, myseg)	# compute mesh
        for a in grouplist:		# copy groups from geometry
            hexa.GroupOnGeom(a)
        meshlist.append(hexa)			# collect submesh in list	
        obc = obc + 1
        
        y0 = y0 - cw * 2.0 


    ### now join all meshes
    MyCompound = smesh.Concatenate( meshlist, True, True, tolerance, False, name='Ribs 2')
    #                              meshlist, unite groups, merge nodes, tolerance, force groups creation, name
    
    ### join all subshapes
    mygeo = geompy.MakePartition( geolist )	
    geompy.addToStudy(mygeo, 'Ribs 2')
    
    # prepare for cleaning up the study
    for a in meshlist:
        allmeshlist.append(a)
        
    for a in geolist:
        allgeolist.append(a)

    return MyCompound