"""   Subunit:  Layer1Rib

 - creates channels for a straight channel flow field
 - additional length of channel for inlet / outlet is possible
 - basic shapes (objects) are loaded from separate files
 - intermediate shapes are removed from the study
   (see end of this file, can be commented out in order to display all subshapes for debugging)
 - assigns face groups and one volume group
 - requires: Box.py 

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

#############################################################################################
### FUNCTION to make geometry   -> with inlet / outlet region
#############################################################################################
def CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolname,  allgeolist, allmeshlist):
	#: Create the serpentine 
	# info:  if subshapes are not on display in the SALOME study, the face groups will not be copied to the mesh
	x0 = inputgeo[0]
	y0 = inputgeo[1]
	z0 = inputgeo[2]
	cw = inputgeo[3] 	# channel width 
	ch = inputgeo[4]	# channel height
	rw = inputgeo[3] / inputgeo[6] # rib width
	chan = inputgeo[8]
	mygroups = myfacegroups
	
	geolist = []	
	meshlist = []
	mybox = []
	obc = 0 # object counter
	mydisplayname = 'Layer1Rib '
	segdelta = 0.01   # this is added in the integer conversion of number of segments (mesh) for safety reasons
	
	## load basic geometry modules ##
	import Box
	
	#################################
	### here are some internal lengths for the overall design
	myres = float( inputmesh[0] / cw)	# mesh resolution
	
	"""
	Flow field has ribs to the left and to the right.
	Channel flow in y-direction.
	"""
	
	######################
	### Main      zone ###
	######################
	
	y0 = y0 + inputgeo[9]
	num = 0 #  counter for channels
	mylen = inputgeo[5]
	seglen = int( myres * mylen / inputmesh[1] + segdelta) # segments in flow direction (y-direction)
	
	while num < ( chan + 1 ):
		num = num + 1
		
		mybox = []
		Box.DefPts(geompy, math, rw, mylen , ch, x0, y0, z0, 0, mybox)
		myobject = Box.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		if num == 1:
			faces = [4, 1, 4, 4, 4, -1]
		elif num > chan:
			faces = [4, 1, 4, 4, -1, 4]
		else:
			faces = [4, 1, 4, 4, -1, -1]
		grouplist = Box.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		Box.Mesh(geompy, mybox, myobject, hexa, inputmesh[0], inputmesh[3], seglen)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		
		obc = obc + 1
		x0 = x0 + rw + cw
	
	
	### join all subshapes
	mygeo = geompy.MakePartition( geolist )	
	geompy.addToStudy(mygeo, 'Layer1Rib')
	#geolist.append(mygeo)
	
	### now join all meshes
	MyCompound = smesh.Concatenate( meshlist, 1, True, 1e-05, False, name='Ribs1')
	
	# garbage collection
	for a in meshlist:
		allmeshlist.append(a)
		
	for a in geolist:
		allgeolist.append(a)
		
	
	return MyCompound
#############################################################################################
### END of FUNCTIONS
#############################################################################################